@@ -6,7 +6,7 @@ from typing import TYPE_CHECKING
 from e3.anod.spec import Anod
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, Final, List, Literal, Optional, Tuple, Union
+    from typing import Any, Final, Literal, Optional, Union
     from e3.anod.package import Source, SourceBuilder
     from e3.collection.dag import DAG
 
@@ -70,11 +70,11 @@ class GetSource(Action):
 
         :param builder: A SourceBuilder object for the source we need to get.
         """
-        super().__init__(uid="source_get.%s" % builder.name, data=builder)
+        super().__init__(uid=f"source_get.{builder.name}", data=builder)
         self.builder = builder
 
     def __str__(self) -> str:
-        return "get source %s" % self.data.name
+        return f"get source {self.data.name}"
 
 
 class Download(Action):
@@ -98,11 +98,11 @@ class DownloadSource(Download):
         :param builder: A SourceBuilder object for the source we need
             to download.
         """
-        super().__init__(uid="download.%s" % builder.name, data=builder)
+        super().__init__(uid=f"download.{builder.name}", data=builder)
         self.builder = builder
 
     def __str__(self) -> str:
-        return "download source %s" % self.builder.name
+        return f"download source {self.builder.name}"
 
 
 class InstallSource(Action):
@@ -126,7 +126,7 @@ class InstallSource(Action):
         self.source = source
 
     def __str__(self) -> str:
-        return "install source %s" % self.data[1].name
+        return f"install source {self.data[1].name}"
 
 
 class CreateSource(Action):
@@ -146,13 +146,14 @@ class CreateSource(Action):
         :param source_name: name of source package to assemble
         """
         super().__init__(
-            uid=f"{anod_instance.uid}.{source_name}", data=(anod_instance, source_name),
+            uid=f"{anod_instance.uid}.{source_name}",
+            data=(anod_instance, source_name),
         )
         self.anod_instance = anod_instance
         self.source_name = source_name
 
     def __str__(self) -> str:
-        return "create source %s" % self.data[1]
+        return f"create source {self.data[1]}"
 
 
 class CreateSources(Action):
@@ -170,11 +171,11 @@ class CreateSources(Action):
 
         :param anod_instance: the Anod instance of the spec
         """
-        super().__init__(uid="%s.sources" % anod_instance.uid, data=(anod_instance))
+        super().__init__(uid=f"{anod_instance.uid}.sources", data=(anod_instance))
         self.anod_instance = anod_instance
 
     def __str__(self) -> str:
-        return "create all sources for %s.anod" % self.anod_instance.name
+        return f"create all sources for {self.anod_instance.name}.anod"
 
 
 class Checkout(Action):
@@ -186,7 +187,7 @@ class Checkout(Action):
 
     __slots__ = ("uid", "repo_name", "repo_data")
 
-    def __init__(self, repo_name: str, repo_data: Dict[str, str]):
+    def __init__(self, repo_name: str, repo_data: dict[str, str]):
         """Initialize a Checkout object.
 
         :param repo_name: The name of the repository.
@@ -194,14 +195,15 @@ class Checkout(Action):
             - 'url': The repository URL;
             - 'revision': The revision to checkout;
             - 'vcs': The Version Control System kind (a string).
-                At present, only 'git' is supported.
+
+        At present, only 'git' is supported.
         """
-        super().__init__(uid="checkout.%s" % repo_name, data=(repo_name, repo_data))
+        super().__init__(uid=f"checkout.{repo_name}", data=(repo_name, repo_data))
         self.repo_name = repo_name
         self.repo_data = repo_data
 
     def __str__(self) -> str:
-        return "checkout %s" % self.data[0]
+        return f"checkout {self.data[0]}"
 
 
 class AnodAction(Action):
@@ -223,10 +225,12 @@ class AnodAction(Action):
 
     def __str__(self) -> str:
         result = "{} {} for {}".format(
-            self.data.kind, self.data.name, self.data.env.platform,
+            self.data.kind,
+            self.data.name,
+            self.data.env.platform,
         )
         if self.data.qualifier:
-            result += " (qualifier=%s)" % self.data.qualifier
+            result += f" (qualifier={self.data.qualifier})"
         return result
 
 
@@ -267,7 +271,7 @@ class DownloadBinary(Download):
         super().__init__(uid=uid, data=data)
 
     def __str__(self) -> str:
-        return "download binary of %s" % self.uid.split(".", 1)[1].rsplit(".", 1)[0]
+        return "download binary of %s" % self.uid.rsplit(".", 1)[0]
 
 
 class Upload(Action):
@@ -298,7 +302,8 @@ class UploadComponent(Upload):
 
     def __str__(self) -> str:
         return "upload {} of {}".format(
-            self.str_prefix, self.uid.split(".", 1)[1].rsplit(".", 1)[0],
+            self.str_prefix,
+            self.uid.rsplit(".", 1)[0],
         )
 
 
@@ -336,7 +341,7 @@ class UploadSource(Upload):
 
     def __str__(self) -> str:
         """Return string representation."""
-        return "upload source %s" % self.source_name
+        return f"upload source {self.source_name}"
 
 
 class Decision(Action, metaclass=abc.ABCMeta):
@@ -416,7 +421,7 @@ class Decision(Action, metaclass=abc.ABCMeta):
         self.expected_choice: Optional[Choice] = None
         self.left_action = left
         self.right_action = right
-        self.triggers: List[Tuple[str, Choice, str]] = []
+        self.triggers: list[tuple[str, Choice, str]] = []
         self.decision_maker: Optional[str] = None
 
     @property
