@@ -13,7 +13,7 @@ logger = e3.log.getLogger("buildspace")
 
 
 if TYPE_CHECKING:
-    from typing import Optional, List
+    from typing import Optional
 
 
 class BuildSpace:
@@ -29,7 +29,6 @@ class BuildSpace:
         "src",
         "test",
         "tmp",
-        "src",
     )
 
     def __init__(self, root_dir: str):
@@ -62,23 +61,23 @@ class BuildSpace:
         return True
 
     @property
-    def dirs(self) -> List[str]:
+    def dirs(self) -> list[str]:
         return list(self.directory_mapping.values())
 
     def subdir(self, name: str) -> str:
         if name not in self.DIRS:
-            raise ValueError("invalid subdir %s" % name)
+            raise ValueError(f"invalid subdir {name}")
         return os.path.join(self.root_dir, self.directory_mapping[name])
 
     def __getattr__(self, name: str) -> str:
         if name.endswith("_dir") and name[:-4] in self.DIRS:
             return self.subdir(name[:-4])
-        raise AttributeError("unknown build space attribute: %s" % name)
+        raise AttributeError(f"unknown build space attribute: {name}")
 
     def __getitem__(self, key: str) -> str:
         if key.isupper() and key.endswith("_DIR"):
             return getattr(self, key.lower(), None)
-        raise KeyError("invalid build space key: %s" % key)
+        raise KeyError(f"invalid build space key: {key}")
 
     def create(self, quiet: bool = False) -> None:
         """Create a build space.
@@ -99,7 +98,7 @@ class BuildSpace:
 
         self.initialized = True
 
-    def reset(self, keep: Optional[List[str]] = None) -> None:
+    def reset(self, keep: Optional[list[str]] = None) -> None:
         """Reset build space.
 
         The function delete the complete buildspace. The only elements that
