@@ -9,13 +9,16 @@ from typing import TYPE_CHECKING
 
 import e3.log
 from e3.anod.status import ReturnValue
-from e3.os.process import PIPE
+from e3.os.fs import unixpath, which
+from e3.os.process import PIPE, Run
 from e3.fs import VCS_IGNORE_LIST, get_filetree_state, rm, sync_tree
 from e3.vcs.git import GitError, GitRepository
 from e3.vcs.svn import SVNError, SVNRepository
 
 if TYPE_CHECKING:
-    from typing import Callable, List, Optional, Tuple
+    from typing import Literal, Optional
+    from collections.abc import Callable
+    from e3.mypy import assert_never
 
 logger = e3.log.getLogger("e3.anod.checkout")
 
@@ -52,19 +55,30 @@ class CheckoutManager:
         self.metadata_file = self.working_dir + "_checkout.json"
         self.changelog_file = self.working_dir + "_changelog.json"
 
-    def update(self, vcs: str, url: str, revision: Optional[str] = None) -> ReturnValue:
+    def update(
+        self,
+        vcs: Literal["git"] | Literal["svn"] | Literal["external"],
+        url: str,
+        revision: Optional[str] = None,
+    ) -> ReturnValue:
         """Update content of the working directory.
 
-        :param vcs: vcd kind
-        :param url: repository url
+        :param vcs: vcs kind
+        :param url: repository url, when vcs is external the url is the path
+             to the source directory
         :param revision: revision
+
+        Note that when vcs is set to git or svn, the version control ignore
+        setting is taken into account. Additionally, when the vcs is
+        external and the source directory contains a .git subdirectory then
+        git ignore setting is taken into account.
         """
         # Reset changelog file
         if os.path.isfile(self.changelog_file):
             rm(self.changelog_file)
 
         update: Callable[
-            [str, Optional[str]], Tuple[ReturnValue, Optional[str], Optional[str]]
+            [str, Optional[str]], tuple[ReturnValue, Optional[str], Optional[str]]
         ]
         if vcs == "git":
             update = self.update_git
@@ -73,8 +87,7 @@ class CheckoutManager:
         elif vcs == "external":
             update = self.update_external
         else:
-            logger.error("Invalid repository type: %s" % vcs)
-            return ReturnValue.failure
+            assert_never()
 
         result, old_commit, new_commit = update(url=url, revision=revision)
 
@@ -93,49 +106,83 @@ class CheckoutManager:
 
     def update_external(
         self, url: str, revision: Optional[str]
-    ) -> Tuple[ReturnValue, str, str]:
+    ) -> tuple[ReturnValue, str, str]:
         """Update working dir using a local directory.
 
         :param url: path to the repository
         :param revision: ignored
+
+        If <url>/.git is a directory then git ls-files will be called to get
+        the list of files to ignore.
         """
         if os.path.isdir(self.working_dir):
             old_commit = get_filetree_state(self.working_dir)
         else:
             old_commit = ""
-        ignore_list: List[str] = []
+        ignore_list: list[str] = []
 
-        if os.path.isdir(os.path.join(url, ".git")):
-            # It seems that this is a git repository. Get the list of files to
-            # ignore
-            try:
-                g = GitRepository(working_tree=url)
-                ignore_list_lines = g.git_cmd(
-                    [
-                        "ls-files",
-                        "-o",
-                        "--ignored",
-                        "--exclude-standard",
-                        "--directory",
-                    ],
-                    output=PIPE,
-                ).out
-                ignore_list = [
-                    "/%s" % f.strip().rstrip("/")
-                    for f in ignore_list_lines.splitlines()
-                ]
-                logger.debug("Ignore in external: %s", ignore_list)
-            except Exception:
-                # don't crash on exception
-                pass
+        if which("rsync") and "use-rsync" in os.environ.get(
+            "E3_ENABLE_FEATURE", ""
+        ).split(","):
+            # Run rsync using -a but without preserving timestamps. --update switch
+            # is also used to skip files that are older in the user directory than
+            # in the checkout itself. This ensure rsync remain efficient event when
+            # timestamps are not preserved (otherwise rsync has to compute checksum
+            # of all file as quick check cannot be used when timestamp is not
+            # preserved).
+            rsync_cmd = [
+                "rsync",
+                "--update",
+                "-rlpgoD",
+                f"{unixpath(url)}/",
+                f"{unixpath(self.working_dir)}",
+                "--delete-excluded",
+            ] + [f"--exclude={el}" for el in VCS_IGNORE_LIST]
 
-        sync_tree(
-            url,
-            self.working_dir,
-            preserve_timestamps=False,
-            delete_ignore=True,
-            ignore=list(VCS_IGNORE_LIST) + ignore_list,
-        )
+            if os.path.isdir(os.path.join(url, ".git")) and os.path.isfile(
+                os.path.join(url, ".gitignore")
+            ):
+                rsync_cmd.append("--filter=:- .gitignore")
+
+            p = Run(rsync_cmd, cwd=url, output=None)
+            if p.status != 0:
+                raise e3.error.E3Error("rsync failed")
+        else:
+            # Test for a git repository by looking for ".git" in the current
+            # directory - either as a file or as a directory. We check both
+            # because, if a normal git clone, ".git" is a directory; if a
+            # git clone with `git submodule init`, ".git" is a file.
+            if os.path.exists(os.path.join(url, ".git")):
+                # It seems that this is a git repository. Get the list of files to
+                # ignore
+                try:
+                    g = GitRepository(working_tree=url)
+                    ignore_list_lines = g.git_cmd(
+                        [
+                            "ls-files",
+                            "-o",
+                            "--ignored",
+                            "--exclude-standard",
+                            "--directory",
+                        ],
+                        output=PIPE,
+                    ).out
+                    ignore_list = [
+                        f"/{f.strip().rstrip('/')}"
+                        for f in ignore_list_lines.splitlines()
+                    ]
+                    logger.debug("Ignore in external: %s", ignore_list)
+                except Exception:  # defensive code
+                    # don't crash on exception
+                    pass
+
+            sync_tree(
+                url,
+                self.working_dir,
+                preserve_timestamps=False,
+                delete_ignore=True,
+                ignore=list(VCS_IGNORE_LIST) + ignore_list,
+            )
 
         new_commit = get_filetree_state(self.working_dir)
         if new_commit == old_commit:
@@ -143,9 +190,18 @@ class CheckoutManager:
         else:
             return ReturnValue.success, old_commit, new_commit
 
+    @staticmethod
+    def git_remote_name(url: str) -> str:
+        """Return the remote name computed for an url.
+
+        :param url: the git url
+        :return: the remote name
+        """
+        return hashlib.sha256(url.encode("utf-8")).hexdigest()
+
     def update_git(
         self, url: str, revision: Optional[str]
-    ) -> Tuple[ReturnValue, Optional[str], Optional[str]]:
+    ) -> tuple[ReturnValue, Optional[str], Optional[str]]:
         """Update working dir using a Git repository.
 
         :param url: git repository url
@@ -163,7 +219,8 @@ class CheckoutManager:
         # This ensure that when the url does not change, git will not
         # redownload all the objects on each objects (and thus avoid
         # disk space leaks).
-        remote_name = hashlib.sha256(url.encode("utf-8")).hexdigest()
+        remote_name = self.git_remote_name(url)
+
         g.init()
 
         # Do the remote addition manually as in that context we can ignore
@@ -172,7 +229,7 @@ class CheckoutManager:
             remote_list = g.git_cmd(["remote"], output=PIPE).out.splitlines()
             if remote_name not in remote_list:
                 g.git_cmd(["remote", "add", remote_name, url])
-        except Exception:
+        except Exception:  # defensive code
             # Ignore exception as it probably means that remote already exist
             # In case of real error the failure will be detected later.
             pass
@@ -181,7 +238,19 @@ class CheckoutManager:
             old_commit = g.rev_parse()
 
             # Using fetch + checkout ensure caching is effective
-            g.git_cmd(["fetch", "-f", remote_name, "%s:refs/e3-checkout" % revision])
+            shallow = "git_shallow_fetch" in os.environ.get(
+                "E3_ENABLE_FEATURE", ""
+            ).split(",") and (not self.compute_changelog or not old_commit)
+            g.git_cmd(
+                [
+                    "fetch",
+                    "-f",
+                    "--depth=1" if shallow else None,
+                    remote_name,
+                    f"{revision}:refs/e3-checkout",
+                ]
+            )
+
             g.checkout("refs/e3-checkout", force=True)
             new_commit = g.rev_parse()
 
@@ -189,15 +258,18 @@ class CheckoutManager:
             p = g.git_cmd(["status", "--porcelain"], output=PIPE)
             if p.out:
                 logger.error(
-                    "Repository %s is locally modified, saving " "diff in stash\n%s",
+                    "Repository %s is locally modified, saving diff in stash\n%s",
                     self.name,
                     p.out,
                 )
                 g.git_cmd(["stash", "save", "-u"])
 
-            if old_commit == new_commit:
-                result = ReturnValue.unchanged
-            elif self.compute_changelog:
+            result = (
+                ReturnValue.unchanged
+                if old_commit == new_commit
+                else ReturnValue.success
+            )
+            if self.compute_changelog:
                 # Fetch the change log and dump it into the changelog file
                 with closing(tempfile.NamedTemporaryFile(mode="w", delete=False)) as fd:
                     g.write_log(fd, rev_range=f"{old_commit}..{new_commit}")
@@ -210,19 +282,15 @@ class CheckoutManager:
 
                 with open(self.changelog_file, "w") as fd:
                     json.dump(commits, fd)
-                # We have removed local changes or updated the git repository
-                result = ReturnValue.success
-            else:
-                result = ReturnValue.success
 
         except GitError:
-            logger.exception("Error during git update %s" % self.name)
+            logger.exception(f"Error during git update {self.name}")
             result = ReturnValue.failure
         return result, old_commit, new_commit
 
     def update_svn(
         self, url: str, revision: Optional[str]
-    ) -> Tuple[ReturnValue, Optional[str], Optional[str]]:
+    ) -> tuple[ReturnValue, Optional[str], Optional[str]]:
         """Update working dir using a SVN repository.
 
         :param url: git repository url
@@ -238,7 +306,7 @@ class CheckoutManager:
                 old_commit = working_copy.current_revision
             except SVNError:
                 logger.error(
-                    "Unable to get SVN informations form the %s " "working dir",
+                    "Unable to get SVN informations form the %s working dir",
                     self.name,
                 )
         try:
@@ -253,7 +321,7 @@ class CheckoutManager:
             new_commit = working_copy.current_revision
             if local_change_detected:
                 logger.error(
-                    "Repository %s was locally modified," " clean done.", self.name
+                    "Repository %s was locally modified, clean done.", self.name
                 )
             if old_commit == new_commit:
                 result = ReturnValue.unchanged
