@@ -13,7 +13,7 @@ from e3.anod.spec import Anod, parse_command
 from e3.os.fs import unixpath
 
 if TYPE_CHECKING:
-    from typing import Dict, List, Optional, Tuple, Union
+    from typing import Optional
     from e3.os.process import Run
 
 log = e3.log.getLogger("anod.helpers")
@@ -28,6 +28,7 @@ class Make:
         makefile: Optional[str] = None,
         exec_dir: Optional[str] = None,
         jobs: Optional[int] = None,
+        make_exe: str = "make",
     ):
         """Initialize a Make object.
 
@@ -36,6 +37,7 @@ class Make:
         :param exec_dir: path to the directory from where the make should be
             called. If None use the anod instance buildspace build dir.
         :param jobs: number of jobs to run in parallel
+        :param make: make executable to use (by default make on the path)
         """
         self.anod_instance = anod_instance
         self.exec_dir = exec_dir
@@ -46,10 +48,11 @@ class Make:
         self.jobs = jobs
         if jobs is None:
             self.jobs = anod_instance.jobs
-        self.var_list = {}  # type: Dict[str, str]
+        self.var_list = {}  # type: dict[str, str]
         self.default_target = None  # type: Optional[str]
+        self.make_exe = make_exe
 
-    def set_var(self, name: str, value: Union[str, List[str]]) -> None:
+    def set_var(self, name: str, value: str | list[str]) -> None:
         """Set a Make variable.
 
         :param name: name of the variable
@@ -92,11 +95,11 @@ class Make:
 
     def cmdline(
         self,
-        target: Optional[Union[str, List[str]]] = None,
+        target: Optional[str | list[str]] = None,
         jobs: Optional[int] = None,
         exec_dir: Optional[str] = None,
         timeout: Optional[float] = None,
-    ) -> Dict[str, Union[List[str], dict]]:
+    ) -> dict[str, list[str] | dict]:
         """Return the make command line.
 
         :param target: optional target or list of targets to use
@@ -109,12 +112,12 @@ class Make:
            - cmd: containing the command line to pass to gnatpython.ex.Run
            - options: options to pass to gnatpython.ex.Run
         """
-        cmd_arg_list = ["make"]
+        cmd_arg_list = [self.make_exe]
 
         if self.makefile is not None:
             cmd_arg_list += ["-f", unixpath(self.makefile)]
 
-        cmd_arg_list += ["-j", "%s" % str(jobs) if jobs is not None else str(self.jobs)]
+        cmd_arg_list += ["-j", str(jobs) if jobs is not None else str(self.jobs)]
 
         for key in self.var_list:
             cmd_arg_list.append("{}={}".format(key, self.var_list[key]))
@@ -167,7 +170,7 @@ class Configure:
         self.exec_dir = exec_dir
         if self.exec_dir is None:
             self.exec_dir = self.anod_instance.build_space.build_dir
-        self.args = []  # type: List[str]
+        self.args = []  # type: list[str]
 
         # Value of the --target, --host and --build arguments
         self.target = None
@@ -255,8 +258,8 @@ class Configure:
 
 
 def text_replace(
-    filename: str, pattern: List[Tuple[Union[bytes, str], Union[bytes, str]]]
-) -> List[int]:
+    filename: str, pattern: list[tuple[bytes | str, bytes | str]]
+) -> list[int]:
     """Replace patterns in a file.
 
     :param filename: file path
