@@ -5,7 +5,8 @@ import collections
 from typing import overload, TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple
+    from typing import Any, NamedTuple, Optional
+    from collections.abc import Callable
     from e3.anod.spec import Anod
     from e3.anod.context import AnodContext
     from e3.anod.package import SourceBuilder
@@ -20,7 +21,6 @@ if TYPE_CHECKING:
         track: bool
         has_closure: bool
 
-
 else:
     SourceKey = collections.namedtuple("SourceKey", ["anod_uid", "src_name", "publish"])
     PackageKey = collections.namedtuple(
@@ -113,8 +113,8 @@ class SourceClosure:
         self.anod_instance = anod_instance
         self.expand_packages = expand_packages
         self.context = context
-        self.source_list: Dict[SourceKey, Optional[Any]] = {}
-        self.package_list: Dict[PackageKey, Optional[Any]] = {}
+        self.source_list: dict[SourceKey, Optional[Any]] = {}
+        self.package_list: dict[PackageKey, Optional[Any]] = {}
         self.compute_closure(self.anod_instance, publish=True)
         self.data_key = data_key
         if self.data_key is None:
@@ -135,7 +135,7 @@ class SourceClosure:
 
         # Otherwise consider the sources and recursively iterate on the
         # dependencies.
-        for source in getattr(spec, "%s_source_list" % spec.kind, []):
+        for source in getattr(spec, f"{spec.kind}_source_list", []):
             publish_source = publish and source.publish
             self.source_list[SourceKey(spec.uid, source.name, publish_source)] = None
 
@@ -155,7 +155,7 @@ class SourceClosure:
                         )
                 self.compute_closure(dep_spec, publish and dep.track)
 
-    def resolve_package(self, spec_uid: str, data: List[Tuple[Any, bool]]) -> None:
+    def resolve_package(self, spec_uid: str, data: list[tuple[Any, bool]]) -> None:
         """Associate source information to a given package.
 
         :param spec_uid: the anod uid
@@ -180,7 +180,7 @@ class SourceClosure:
             if src_key.src_name == source_name:
                 self.source_list[src_key] = data
 
-    def get_source_list(self) -> List[List[Any]]:
+    def get_source_list(self) -> list[list[Any]]:
         """Get the closure source list.
 
         The function return the list of data for the sources in the closure.
@@ -191,7 +191,7 @@ class SourceClosure:
 
         :return: a list of list (source, publish)
         """
-        result: Dict[str, List[Any]] = {}
+        result: dict[str, list[Any]] = {}
         for src_key, src in list(self.source_list.items()):
             assert src is not None, "missing resolution"
             if TYPE_CHECKING:
