@@ -10,7 +10,6 @@ from typing import TYPE_CHECKING
 import e3.log
 import e3.os.process
 from e3.anod.buildspace import BuildSpace
-from e3.anod.error import SandBoxError
 from e3.env import Env
 from e3.fs import mkdir, rm
 from e3.os.fs import chmod
@@ -19,12 +18,12 @@ logger = e3.log.getLogger("sandbox")
 
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, Optional
+    from typing import Any, Optional
 
 
 class SandBox:
-    def __init__(self) -> None:
-        self.__root_dir: Optional[str] = None
+    def __init__(self, root_dir: str) -> None:
+        self.root_dir: str = os.path.realpath(root_dir)
         self.build_id: Optional[str] = None
         self.build_date: Optional[str] = None
         self.build_version: Optional[str] = None
@@ -40,20 +39,43 @@ class SandBox:
             "vcs",
             "patch",
         )
-
-        self.meta_dir: Optional[str] = None
-        self.tmp_dir: Optional[str] = None
-        self.tmp_cache_dir: Optional[str] = None
-        self.src_dir: Optional[str] = None
-        self.log_dir: Optional[str] = None
-        self.vcs_dir: Optional[str] = None
-        self.patch_dir: Optional[str] = None
-        self.bin_dir: Optional[str] = None
-        self.__specs_dir: Optional[str] = None
+        self.meta_dir: str = ""
+        self.tmp_dir: str = ""
+        self.tmp_cache_dir: str = ""
+        self.src_dir: str = ""
+        self.log_dir: str = ""
+        self.vcs_dir: str = ""
+        self.patch_dir: str = ""
+        self.bin_dir: str = ""
         self.is_alternate_specs_dir = False
+        self.__specs_dir = os.path.join(self.root_dir, "specs")
 
         # Contains the loaded version of user.yaml if present
-        self.user_config: Optional[Dict[str, Any]] = None
+        self.user_config: Optional[dict[str, Any]] = None
+
+        # For each directory create an attribute containing its path
+        for d in self.dirs:
+            setattr(
+                self,
+                (f"{d}_dir").replace(os.path.sep, "_"),
+                os.path.join(self.root_dir, d),
+            )
+
+        # specs_dir path can be changed by a configuration in user.yaml
+        user_yaml = os.path.join(self.root_dir, "user.yaml")
+        if os.path.exists(user_yaml):
+            with open(user_yaml) as f:
+                self.user_config = yaml.safe_load(f)
+
+            # Accept both specs_dir and module_dir key (in that order) to
+            # get the path to the anod specification files
+            specs_dir = (
+                self.user_config.get("specs_dir", self.user_config.get("module_dir"))
+                if self.user_config is not None
+                else None
+            )
+            if specs_dir is not None:
+                self.specs_dir = specs_dir
 
         self.default_env = {
             "LANG": "C",
@@ -79,51 +101,7 @@ class SandBox:
             os.environ[k] = v
 
     @property
-    def root_dir(self) -> str:
-        """Root path of the sandbox.
-
-        :raise SandBoxError: when the sandbox is not initialized
-        """
-        if self.__root_dir is None:
-            raise SandBoxError(
-                origin="root_dir", message="sandbox not loaded. Please call load()"
-            )
-        return self.__root_dir
-
-    @root_dir.setter
-    def root_dir(self, d: str) -> None:
-        new_dir = os.path.realpath(d)
-        if new_dir == self.__root_dir:
-            return  # nothing to do
-
-        self.__root_dir = new_dir
-
-        # For each directory create an attribute containing its path
-        for d in self.dirs:
-            setattr(
-                self, ("%s_dir" % d).replace(os.path.sep, "_"), os.path.join(new_dir, d)
-            )
-
-        # specs_dir path can be changed by a configuration in user.yaml
-        user_yaml = os.path.join(new_dir, "user.yaml")
-        if os.path.exists(user_yaml):
-            with open(user_yaml) as f:
-                self.user_config = yaml.safe_load(f)
-
-            # Accept both specs_dir and module_dir key (in that order) to
-            # get the path to the anod specification files
-            specs_dir = (
-                self.user_config.get("specs_dir", self.user_config.get("module_dir"))
-                if self.user_config is not None
-                else None
-            )
-            if specs_dir is not None:
-                self.specs_dir = specs_dir
-        else:
-            self.__specs_dir = os.path.join(new_dir, "specs")
-
-    @property
-    def specs_dir(self) -> Optional[str]:
+    def specs_dir(self) -> str:
         """Return where to find anod specification files."""
         return self.__specs_dir
 
@@ -140,7 +118,7 @@ class SandBox:
     def create_dirs(self) -> None:
         """Create all required sandbox directories."""
         for d in self.dirs:
-            mkdir(getattr(self, ("%s_dir" % d).replace(os.path.sep, "_")))
+            mkdir(getattr(self, f"{d}_dir".replace(os.path.sep, "_")))
 
     def get_build_space(self, name: str, platform: Optional[str] = None) -> BuildSpace:
         """Get build space.
@@ -157,7 +135,6 @@ class SandBox:
     def dump_configuration(self) -> None:
         # Compute command line for call to e3-sandbox create. Ensure that the
         # paths are made absolute (path to sandbox, script).
-        assert self.meta_dir is not None
         cmd_line = [sys.executable, os.path.abspath(__file__)]
         cmd_line += sys.argv[1:]
         sandbox_conf = os.path.join(self.meta_dir, "sandbox.yaml")
@@ -165,7 +142,6 @@ class SandBox:
             yaml.safe_dump({"cmd_line": cmd_line}, f)
 
     def get_configuration(self) -> dict:
-        assert self.meta_dir is not None
         sandbox_conf = os.path.join(self.meta_dir, "sandbox.yaml")
         with open(sandbox_conf) as f:
             return yaml.safe_load(f)
@@ -173,18 +149,16 @@ class SandBox:
     def write_scripts(self) -> None:
         from setuptools.command.easy_install import get_script_args
 
-        assert self.bin_dir is not None
-
         # Retrieve sandbox_scripts entry points
         e3_distrib = get_distribution("e3-core")
 
         class SandboxDist:
-            def get_entry_map(self, group):
+            def get_entry_map(self, group):  # type: ignore
                 if group != "console_scripts":
                     return {}
                 return e3_distrib.get_entry_map("sandbox_scripts")
 
-            def as_requirement(self):
+            def as_requirement(self):  # type: ignore
                 return e3_distrib.as_requirement()
 
         for script in get_script_args(dist=SandboxDist()):
