@@ -11,18 +11,10 @@ from e3.error import E3Error
 if TYPE_CHECKING:
     from typing import (
         Any,
-        Callable,
-        Dict,
-        FrozenSet,
         Hashable,
-        List,
-        Iterator,
         Optional,
-        Sequence,
-        Set,
-        Tuple,
-        Union,
     )
+    from collections.abc import Callable, Iterator, Sequence
 
     VertexID = Hashable
 
@@ -60,7 +52,7 @@ class DAGIterator:
     def __iter__(self) -> DAGIterator:
         return self
 
-    def __next__(self) -> Union[Tuple[None, None], Tuple[VertexID, Any]]:
+    def __next__(self) -> tuple[None, None] | tuple[VertexID, Any]:
         """Retrieve next_element with with_predecessors=False.
 
         The intermediate function is needed in Python 3.x
@@ -70,7 +62,9 @@ class DAGIterator:
             return (None, None)
         return (vertex_id, data)
 
-    def next_element(self,) -> Tuple[Union[None, VertexID], Any, FrozenSet[VertexID]]:
+    def next_element(
+        self,
+    ) -> tuple[Optional[VertexID], Any, frozenset[VertexID]]:
         """Retrieve next element in topological order.
 
         :return: a vertex id, data, predecessors. (None, None, frozenset()) is
@@ -138,15 +132,24 @@ class DAG:
 
     def __init__(self) -> None:
         """Initialize a DAG."""
-        self.vertex_data: Dict[VertexID, Any] = {}
-        self.tags: Dict[VertexID, Any] = {}
+        self.vertex_data: dict[VertexID, Any] = {}
+        self.tags: dict[VertexID, Any] = {}
 
-        self.__vertex_predecessors: Dict[VertexID, FrozenSet[VertexID]] = {}
-        self.__vertex_successors: Dict[VertexID, FrozenSet[VertexID]] = {}
+        self.__vertex_predecessors: dict[VertexID, frozenset[VertexID]] = {}
+        self.__vertex_successors: dict[VertexID, frozenset[VertexID]] = {}
         self.__has_cycle: Optional[bool] = None
+        self.__cached_topological_order: Optional[list[tuple[VertexID, Any]]] = None
+
+    def reset_caches(self) -> None:
+        """Reset caches for DAG properties (cycle and cached topological order).
+
+        This is a mandatory step when changing DAG edges.
+        """
+        self.__has_cycle = None
+        self.__cached_topological_order = None
 
     @property
-    def vertex_predecessors(self) -> Dict[VertexID, FrozenSet[VertexID]]:
+    def vertex_predecessors(self) -> dict[VertexID, frozenset[VertexID]]:
         """Return predecessors.
 
         Meant only for backward compatibility. Use vertex_predecessors_items.
@@ -160,19 +163,19 @@ class DAG:
 
     def vertex_predecessors_items(
         self,
-    ) -> Iterator[Tuple[VertexID, FrozenSet[VertexID]]]:
+    ) -> Iterator[tuple[VertexID, frozenset[VertexID]]]:
         """Return predecessors.
 
         :return: a list of (vertex id, predecessors)
         """
         return iter(self.__vertex_predecessors.items())
 
-    def get_predecessors(self, vertex_id: VertexID) -> FrozenSet[VertexID]:
+    def get_predecessors(self, vertex_id: VertexID) -> frozenset[VertexID]:
         """Get set of predecessors for a given vertex."""
         return self.__vertex_predecessors.get(vertex_id, frozenset())
 
     def set_predecessors(
-        self, vertex_id: VertexID, predecessors: FrozenSet[VertexID]
+        self, vertex_id: VertexID, predecessors: frozenset[VertexID]
     ) -> None:
         """Set predecessors for a given vertex.
 
@@ -181,16 +184,16 @@ class DAG:
         self.__vertex_predecessors[vertex_id] = predecessors
         # Reset successors and cycle check results which are now invalid
         self.__vertex_successors = {}
-        self.__has_cycle = None
+        self.reset_caches()
 
-    def get_successors(self, vertex_id: VertexID) -> FrozenSet[VertexID]:
+    def get_successors(self, vertex_id: VertexID) -> frozenset[VertexID]:
         """Get set of successors for a given vertex.
 
         If the global dictionary of vertex successors has not been
         computed or if it has been invalidated then recompute it.
         """
         if self.__vertex_successors == {}:
-            successors: Dict[VertexID, Set[VertexID]] = {
+            successors: dict[VertexID, set[VertexID]] = {
                 k: set() for k in self.__vertex_predecessors
             }
             for pred_k, pred_v in self.__vertex_predecessors.items():
@@ -224,7 +227,7 @@ class DAG:
         max_distance: Optional[int] = None,
         max_element: Optional[int] = None,
         reverse_order: bool = False,
-    ) -> List[Tuple[int, VertexID, Any]]:
+    ) -> list[tuple[int, VertexID, Any]]:
         r"""Get tag context.
 
         Returns the list of predecessors tags along with their vertex id and
@@ -259,7 +262,7 @@ class DAG:
         """
         self.check()
 
-        def get_next(vid: VertexID) -> FrozenSet[VertexID]:
+        def get_next(vid: VertexID) -> frozenset[VertexID]:
             """Get successors or predecessors.
 
             :param vid: vertex id
@@ -270,7 +273,7 @@ class DAG:
                 result = self.get_predecessors(vid)
             return result
 
-        visited: Set[VertexID] = set()
+        visited: set[VertexID] = set()
         tags = []
         distance = 0
         node_tag = self.get_tag(vertex_id)
@@ -311,6 +314,11 @@ class DAG:
     ) -> None:
         """Add a new vertex into the DAG.
 
+        Note that this function always checks that there is no cycle in the
+        current DAG. If you prefer to insert all your node first and then
+        do the check at the end use update_vertex calls followed by a call
+        to check.
+
         :param vertex_id: the name of the vertex
         :param data: data for the vertex.
         :param predecessors: list of predecessors (vertex ids) or None
@@ -318,7 +326,7 @@ class DAG:
         """
         if vertex_id in self.vertex_data:
             raise DAGError(
-                message="vertex %s already exist" % vertex_id, origin="DAG.add_vertex"
+                message=f"vertex {vertex_id} already exist", origin="DAG.add_vertex"
             )
         self.update_vertex(vertex_id, data, predecessors)
 
@@ -327,7 +335,7 @@ class DAG:
         vertex_id: VertexID,
         data: Any = None,
         predecessors: Optional[
-            Union[Sequence[VertexID], Set[VertexID], FrozenSet[VertexID]]
+            Sequence[VertexID] | set[VertexID] | frozenset[VertexID]
         ] = None,
         enable_checks: bool = True,
     ) -> None:
@@ -344,7 +352,7 @@ class DAG:
         :raise: DAGError if cycle is detected
         """
         if predecessors is None:
-            vertex_predecessors: FrozenSet[VertexID] = frozenset()
+            vertex_predecessors: frozenset[VertexID] = frozenset()
         else:
             vertex_predecessors = frozenset(predecessors)
 
@@ -389,13 +397,15 @@ class DAG:
             if data is not None:
                 self.vertex_data[vertex_id] = data
 
-        if not enable_checks:
+        if enable_checks:
+            self.__cached_topological_order = None
+        else:
             # DAG modified without cycle checks, discard cached result
-            self.__has_cycle = None
+            self.reset_caches()
 
     def shortest_path(
         self, source: VertexID, target: VertexID
-    ) -> Optional[List[VertexID]]:
+    ) -> Optional[list[VertexID]]:
         """Compute the shortest path between two vertices of the DAG.
 
         :param source: vertex id of the source
@@ -414,15 +424,15 @@ class DAG:
         infinite = len(self.vertex_data) + 1
 
         # Keep track of minimal distance between vertices and the sources
-        dist: Dict[Optional[VertexID], int] = {k: infinite for k in self.vertex_data}
+        dist: dict[Optional[VertexID], int] = {k: infinite for k in self.vertex_data}
 
         # Keep track of the minimum distance
-        prev: Dict[Optional[VertexID], Optional[VertexID]] = {
+        prev: dict[Optional[VertexID], Optional[VertexID]] = {
             k: None for k in self.vertex_data
         }
 
         # Set of non visited vertices
-        unvisited: Set[Optional[VertexID]] = set(self.vertex_data)
+        unvisited: set[Optional[VertexID]] = set(self.vertex_data)
 
         # The only known distance at startup
         dist[target] = 0
@@ -466,7 +476,7 @@ class DAG:
             # No path exist between source and target (or no cycle).
             return None
         else:
-            result: List[Optional[VertexID]] = [path_source]
+            result: list[Optional[VertexID]] = [path_source]
             while prev[result[-1]] is not None:
                 result.append(prev[result[-1]])
 
@@ -474,7 +484,7 @@ class DAG:
             # is in reality our source node.
             if path_source is None:
                 result[0] = target
-            return result  # type: ignore
+            return result
 
     def check(self) -> None:
         """Check for cycles and inexisting nodes.
@@ -493,43 +503,45 @@ class DAG:
             if len([k for k in preds if k not in self.vertex_data]) > 0:
                 self.__has_cycle = True
                 raise DAGError(
-                    message="invalid nodes in predecessors of %s" % node,
+                    message=f"invalid nodes in predecessors of {node}",
                     origin="DAG.check",
                 )
         # raise DAGError if cycle
+        topological_order = []
         try:
-            for _ in DAGIterator(self):
-                pass
+            for vertex_id, data in DAGIterator(self):
+                if vertex_id is not None:
+                    topological_order.append((vertex_id, data))
         except DAGError:
             self.__has_cycle = True
             raise
         else:
             self.__has_cycle = False
+            self.__cached_topological_order = topological_order
 
-    def get_closure(self, vertex_id: VertexID) -> FrozenSet[VertexID]:
+    def get_closure(self, vertex_id: VertexID) -> set[VertexID]:
         """Retrieve closure of predecessors for a vertex.
 
         :param vertex_id: the vertex to inspect
         :return: a set of vertex_id
         """
-        visited: Set[VertexID] = set()
-        closure = self.get_predecessors(vertex_id)
-        closure_len = len(closure)
+        closure: set[VertexID] = set()
+        to_visit = set(self.get_predecessors(vertex_id))
 
-        while True:
-            for n in closure - visited:
-                visited.add(n)
+        while len(to_visit) > 0:
+            next_visit: set[VertexID] = set()
+            for n in to_visit:
+                closure.add(n)
+                next_visit |= self.get_predecessors(n)
 
-                closure |= self.get_predecessors(n)
+            to_visit = next_visit - closure
 
-            if len(closure) == closure_len:
-                break
-            closure_len = len(closure)
         return closure
 
-    def reverse_graph(self) -> DAG:
+    def reverse_graph(self, enable_checks: bool = True) -> DAG:
         """Compute the reverse DAG.
 
+        :param enable_checks: whether to check that "self" is valid (no cycle)
         :return: the reverse DAG (edge inverted)
         """
         result = DAG()
@@ -543,19 +555,34 @@ class DAG:
             result.update_vertex(node, data=self.vertex_data[node], enable_checks=False)
             for p in predecessors:
                 result.update_vertex(p, predecessors=[node], enable_checks=False)
-        try:
-            result.check()
-        except DAGError:
-            # Check detected
-            self.__has_cycle = True
-            raise
-        else:
-            # No cycle in the DAG
-            self.__has_cycle = False
+
+        # The reversed topological order for "self" is a valid topological
+        # order for "result".
+        result.__has_cycle = self.__has_cycle
+        result.__cached_topological_order = (
+            None
+            if self.__cached_topological_order is None
+            else list(reversed(self.__cached_topological_order))
+        )
+
+        if enable_checks:
+            try:
+                result.check()
+            except DAGError:
+                # Check detected
+                self.__has_cycle = True
+                raise
+            else:
+                self.__has_cycle = False
+
         return result
 
-    def __iter__(self) -> DAGIterator:
-        return DAGIterator(self)
+    def __iter__(self) -> Iterator[tuple[VertexID, Any]]:
+        return (
+            iter(DAGIterator(self))
+            if self.__cached_topological_order is None
+            else iter(self.__cached_topological_order)
+        )
 
     def __contains__(self, vertex_id: VertexID) -> bool:
         """Check if a vertex is present in the DAG."""
@@ -599,13 +626,76 @@ class DAG:
         """
         result = ["digraph G {", 'rankdir="LR";']
         for vertex in self.vertex_data:
-            result.append('"%s"' % vertex)
+            result.append(f'"{vertex}"')
         for vertex, predecessors in self.__vertex_predecessors.items():
             for predecessor in predecessors:
                 result.append(f'"{vertex}" -> "{predecessor}"')
         result.append("}")
         return "\n".join(result)
 
+    def as_tree(self, name_key: Optional[str] = None) -> str:
+        """Return a tree representation of the graph.
+
+        This is similar to the output of the `tree` bash command.
+
+        :param name_key: the data key-value to use as the name of the vertex; if None,
+            the vertex ID is chosen as the name
+        :return: a text tree
+        """
+
+        def get_subtree(
+            root: VertexID, name_key: Optional[str], prefix: str = ""
+        ) -> str:
+            """Get a given subtree starting from a root vertex.
+
+            This is the helper method to help "draw" the text tree.
+            :param root: the root vertex ID.
+            :param name_key: the data key-value to use as the name of the vertex; if
+            None, the vertex ID is chosen as the name
+            :param prefix: the pipe prefix to use for this subtree. This is calculated
+            automatically in the iterations of this method, and should be equal to
+            an empty string for the root vertex.
+            :return: the subtree in text form.
+            """
+            result = f"{root}"
+            if name_key:
+                assert name_key in self[root]
+                result = f"{self[root][name_key]}"
+
+            successors = self.get_successors(root)
+
+            if not successors:
+                # Base case, return node name.
+                return result
+            else:
+                # Recursive case, iterate over children.
+                for i, node in enumerate(sorted(successors)):  # type: ignore
+                    if i == len(successors) - 1:
+                        # If it's the last child, no need to prefix by a pipe.
+                        # Furthermore, the suffix pipe should not continue downwards (we
+                        # need an 'L'-shaped piece instead of a 'T'-shaped one).
+                        new_prefix = "    "
+                        suffix = "\u2514\u2500\u2500 "
+                    else:
+                        # Else, we need to prefix by a pipe.
+                        # Furthermore, we need the suffix pipe to continue downwards
+                        # ('T'-shaped pipe).
+                        new_prefix = "\u2502   "
+                        suffix = "\u251c\u2500\u2500 "
+
+                    node_subtree = get_subtree(
+                        root=node, name_key=name_key, prefix=prefix + new_prefix
+                    )
+                    result += f"\n{prefix}{suffix}{node_subtree}"
+
+                return result
+
+        if not self.__vertex_predecessors:
+            return ""
+        return get_subtree(
+            root=list(self.__vertex_predecessors.keys())[0], name_key=name_key
+        )
+
     def prune(
         self, fun: Callable[[DAG, VertexID], bool], preserve_context: bool = True
     ) -> DAG:
@@ -615,14 +705,14 @@ class DAG:
             pruned. The function receive as parameter the dag and the node id
         :param preserve_context: if True ensure that context is preserved
             (i.e: that calls to get_context will return the same value for
-             both current graph and pruned graph). This means that any attempt
+            both current graph and pruned graph). This means that any attempt
             to remove a node containing a tag will result in a DAGError.
         :return: a new DAG
         """
         result = DAG()
 
         # Used to maintain the new list of predecessors
-        pruned_node_predecessors: Dict[VertexID, Set[VertexID]] = {}
+        pruned_node_predecessors: dict[VertexID, set[VertexID]] = {}
 
         for node, data in self:
             # The new list of predecessors is the union of predecessors of
@@ -638,7 +728,7 @@ class DAG:
             if fun(self, node):
                 # Check if node can pruned
                 if node in self.tags and preserve_context:
-                    raise DAGError("suppressing %s impact context" % node)
+                    raise DAGError(f"suppressing {node} impact context")
 
                 # Node is pruned keep track of its predecessors
                 pruned_node_predecessors[node] = predecessors
@@ -662,5 +752,5 @@ class DAG:
                     "{} -> {}".format(vertex, ", ".join(str(p) for p in predecessors))
                 )
             else:
-                result.append("%s -> (none)" % vertex)
+                result.append(f"{vertex} -> (none)")
         return "\n".join(result)
