@@ -6,7 +6,7 @@ import yaml
 from e3.env import BaseEnv
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, List, Optional
+    from typing import Any, Optional
 
 
 class Host(BaseEnv):
@@ -26,7 +26,7 @@ class Host(BaseEnv):
         :param kwargs: additional user defined data. each key from the data
             dict is accessible like a regular attribute.
         """
-        BaseEnv.__init__(self)
+        super().__init__()
         self.set_build(name=str(platform), version=str(version), machine=str(hostname))
         self._instance.update(kwargs)
 
@@ -44,13 +44,13 @@ class HostDB:
             file. See HostDB.load_yaml_db method for details about the expected
             format
         """
-        self.hosts: Dict[str, Host] = {}
+        self.hosts: dict[str, Host] = {}
 
         if filename is not None:
             self.load_yaml_db(filename)
 
     @property
-    def hostnames(self) -> List[str]:
+    def hostnames(self) -> list[str]:
         """Return the current list of host names.
 
         :return: a list of hostnames
