@@ -13,7 +13,9 @@ from e3.env import BaseEnv
 from e3.error import E3Error
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, List, Optional
+    from types import TracebackType
+    from typing import Any, Optional
+    from collections.abc import Callable
     from e3.collection.toggleable_bool import ToggleableBoolean
     from e3.electrolyt.entry_point import EntryPoint
 
@@ -33,8 +35,8 @@ class Plan:
 
     def __init__(
         self,
-        data: Dict[str, Any],
-        entry_point_cls: Optional[Dict[str, Callable[..., EntryPoint]]] = None,
+        data: dict[str, Any],
+        entry_point_cls: Optional[dict[str, Callable[..., EntryPoint]]] = None,
         plan_ext: str = ".plan",
     ):
         """Initialize a new plan.
@@ -53,7 +55,7 @@ class Plan:
         for k, v in data.items():
             self.mod.__dict__[k] = v
 
-        self.entry_points: Dict[str, EntryPoint] = {}
+        self.entry_points: dict[str, EntryPoint] = {}
 
         if entry_point_cls is None:
             entry_point_cls = {}
@@ -107,12 +109,29 @@ class Plan:
         exec(code, self.mod.__dict__)
 
 
+class PlanActionEnv(BaseEnv):
+    """Store the action environment.
+
+    This includes the build/host/target as well as additional parameters
+    coming from the plan.
+    """
+
+    action: str
+    plan_line: str
+    plan_args: dict[str, Any]
+    plan_call_args: dict[str, Any]
+    push_to_store: bool
+    default_build: bool
+    module: Optional[str]
+    source_packages: Optional[list[str]]
+
+
 class PlanContext:
     """Context in which a Plan is executed."""
 
     def __init__(
         self,
-        stack: Optional[List[BaseEnv]] = None,
+        stack: Optional[list[PlanActionEnv]] = None,
         plan: Optional[Plan] = None,
         ignore_disabled: bool = True,
         server: Optional[BaseEnv] = None,
@@ -120,11 +139,12 @@ class PlanContext:
         host: Optional[str] = None,
         target: Optional[str] = None,
         enabled: bool = True,
+        default_push_to_store: bool = False,
         **kwargs: Any,
     ):
         """Initialize an execution context or a scope.
 
-        :param stack: stack of BaseEnv object that keep track of scopes. Used
+        :param stack: stack of PlanAction object that keep track of scopes. Used
             only internally. User instantiation of PlanContext should be done
             with stack set to None.
         :param plan: the plan to execute
@@ -137,10 +157,14 @@ class PlanContext:
         :param host: see e3.env.BaseEnv.set_env
         :param target: see e3.env.BaseEnv.set_env
         :param enabled: whether the plan line is enabled or disabled
+        :param default_push_to_store: whether pushing packages in the package store
+            is enabled or not, this is the default value and can be overriden
+            by setting push_to_store attribute
         :param kwargs: additional data for the current scope/context
         """
         self.ignore_disabled = ignore_disabled
         if stack:
+            assert server is None, "passing server and stack is not supported"
             # This is a scope creation, so copy current env
             self.stack = stack
             self.plan = plan
@@ -155,12 +179,16 @@ class PlanContext:
             self.stack = []
             if server is not None:
                 # This is a new context
-                new = server.copy(build, host, target)
+                new = PlanActionEnv.from_env(server)
             else:
                 # This is a new context with no server information. In that
                 # case retrieve defaults for the local machine.
-                new = BaseEnv()
+                new = PlanActionEnv()
                 new.set_env(build, host, target)
+            new.push_to_store = default_push_to_store
+            new.default_build = False
+            new.module = None
+            new.source_packages = None
             new.enabled = enabled
 
         # Store additional data
@@ -175,8 +203,8 @@ class PlanContext:
         # scopes. Only initial context use them. The overall scheme
         # works also because all scopes refer to the same stack of env
         # (because the object is mutable).
-        self.actions: Dict[str, Callable] = {}
-        self.action_list: List[BaseEnv] = []
+        self.actions: dict[str, Callable] = {}
+        self.action_list: list[PlanActionEnv] = []
 
     def register_action(self, name: str, fun: Callable) -> None:
         """Register a function that correspond to an action.
@@ -189,7 +217,7 @@ class PlanContext:
         self.actions[name] = fun
 
     @property
-    def env(self) -> BaseEnv:
+    def env(self) -> PlanActionEnv:
         """Get environment for current scope.
 
         :return: the current scope environment
@@ -207,7 +235,7 @@ class PlanContext:
 
     def execute(
         self, plan: Plan, entry_point_name: str, verify: bool = False
-    ) -> List[BaseEnv]:
+    ) -> list[PlanActionEnv]:
         """Execute a plan.
 
         :param plan: the plan to execute
@@ -294,7 +322,7 @@ class PlanContext:
         # coming from environment). For the build, host and target arguments
         # a special processing is done to make the corresponding set_env
         # call on the result BaseEnv object
-        platform: Dict[str, Optional[str]] = {
+        platform: dict[str, Optional[str]] = {
             "build": None,
             "host": None,
             "target": None,
@@ -345,11 +373,18 @@ class PlanContext:
         # Push the action in the current list
         self.action_list.append(result)
 
-    def __enter__(self):
+    def __enter__(self) -> None:
+        assert self.plan is not None
         self.plan.mod.__dict__["env"] = self.env
         return None
 
-    def __exit__(self, _type, _value, _tb):
+    def __exit__(
+        self,
+        _type: Optional[type[BaseException]],
+        _value: Optional[BaseException],
+        _tb: Optional[TracebackType],
+    ) -> None:
         del _type, _value, _tb
         self.stack.pop()
+        assert self.plan is not None
         self.plan.mod.__dict__["env"] = self.env
