@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+from typing import TYPE_CHECKING
+
 import json
 import mimetypes
 import tempfile
@@ -10,18 +12,28 @@ from e3.fs import rm
 from e3.os.process import Run
 from e3.sys import python_script
 
+if TYPE_CHECKING:
+    from typing import Optional
+    from e3.event import Event
+
 
 class S3Handler(EventHandler):
     """Event handler that relies on AWS S3."""
 
-    def __init__(self, event_s3_url, log_s3_url, sse="AES256", profile=None):
+    def __init__(
+        self,
+        event_s3_url: str,
+        log_s3_url: str,
+        sse: str = "AES256",
+        profile: Optional[str] = None,
+    ) -> None:
         self.event_s3_url = event_s3_url
         self.log_s3_url = log_s3_url
         self.aws_profile = profile
         self.sse = sse
 
     @classmethod
-    def decode_config(cls, config_str):
+    def decode_config(cls, config_str: str) -> dict[str, Optional[str]]:
         event_s3_url, log_s3_url, sse, aws_profile = config_str.split(",", 3)
         return {
             "event_s3_url": event_s3_url,
@@ -30,7 +42,7 @@ class S3Handler(EventHandler):
             "profile": aws_profile if aws_profile else None,
         }
 
-    def encode_config(self):
+    def encode_config(self) -> str:
         return "{},{},{},{}".format(
             self.event_s3_url,
             self.log_s3_url,
@@ -38,11 +50,25 @@ class S3Handler(EventHandler):
             self.aws_profile if self.aws_profile is not None else "",
         )
 
-    def send_event(self, event):
-        def s3_cp(from_path, s3_url):
-            cmd = ["s3", "cp", "--sse=%s" % self.sse]
+    def s3_prefix(self, event: Event) -> str:
+        """Additional prefix that depends on the event itself.
+
+        This hook allows a user to add a prefix that depends on the event itself.
+        Note that sufixes are still automatically computed so distinct events can
+        return the same prefix. The final s3 url used will be
+        {log_s3_url}/{s3_prefix}{automatic suffix} for logs and
+        {event_s3_url}/{s3_prefix}{automatic suffix} for events metadata.
+
+        :param event: an event
+        :return: the prefix
+        """
+        return ""
+
+    def send_event(self, event: Event) -> bool:
+        def s3_cp(from_path: str, s3_url: str) -> bool:
+            cmd = ["s3", "cp", f"--sse={self.sse}"]
             if self.aws_profile:
-                cmd.append("--profile=%s" % self.aws_profile)
+                cmd.append(f"--profile={self.aws_profile}")
             cmd += [from_path, s3_url]
 
             s3 = Run(python_script("aws") + cmd, output=None)
@@ -53,7 +79,7 @@ class S3Handler(EventHandler):
         for name, attach in list(event.get_attachments().items()):
             attach_path = attach[0]
             # Push the attachment
-            s3_url = f"{self.log_s3_url}/{event.uid}/{name}"
+            s3_url = f"{self.log_s3_url}/{self.s3_prefix(event)}{event.uid}/{name}"
             success = s3_cp(attach_path, s3_url)
             if not success:
                 return False
@@ -78,7 +104,9 @@ class S3Handler(EventHandler):
             # status. As a consequence the target url in s3 should be different
             # for call to send.
             success = s3_cp(
-                tempfile_name, f"{self.event_s3_url}/{event.uid}-{unique_id()}.s3",
+                tempfile_name,
+                f"{self.event_s3_url}/{self.s3_prefix(event)}"
+                f"{event.uid}-{unique_id()}.s3",
             )
 
             if not success:
