@@ -33,7 +33,7 @@ from e3.fs import get_filetree_state
 from e3.hash import sha256
 
 if TYPE_CHECKING:
-    from typing import Dict, Optional, Set
+    from typing import Optional
 
 logger = e3.log.getLogger("fingerprint")
 
@@ -52,7 +52,7 @@ class Fingerprint:
 
     def __init__(self) -> None:
         """Initialise a new fingerprint instance."""
-        self.elements: Dict[str, str] = {}
+        self.elements: dict[str, str] = {}
         self.add("os_version", Env().build.os.version)
         # ??? add more detailed information about the build machine so that
         # even a minor OS upgrade trigger a rebuild
@@ -70,7 +70,8 @@ class Fingerprint:
             self.elements[name] = value
         else:
             raise E3Error(
-                f"value for {name} should be a string got {value}", "fingerprint.add",
+                f"value for {name} should be a string got {value}",
+                "fingerprint.add",
             )
 
     def add_dir(self, path: str) -> None:
@@ -78,8 +79,10 @@ class Fingerprint:
 
         :param path: a path to a directory
         """
-        assert os.path.isdir(path), "directory %s does not exist" % path
-        self.elements[os.path.basename(path)] = get_filetree_state(path)
+        if os.path.isdir(path):
+            self.elements[os.path.abspath(path)] = get_filetree_state(path)
+        else:
+            self.elements[os.path.abspath(path)] = ""
 
     def add_file(self, filename: str) -> None:
         """Add a file element to the fingerprint.
@@ -90,8 +93,10 @@ class Fingerprint:
         an element for which key is the basename of the file and value is
         is the sha256 of the content
         """
-        assert os.path.isfile(filename), "filename %s does not exist" % filename
-        self.elements[os.path.basename(filename)] = sha256(filename)
+        if os.path.isfile(filename):
+            self.elements[os.path.abspath(filename)] = sha256(filename)
+        else:
+            self.elements[os.path.abspath(filename)] = ""
 
     def __eq__(self, other: object) -> bool:
         """Implement == operator for two fingerprints.
@@ -123,7 +128,7 @@ class Fingerprint:
         """
         return not self == other
 
-    def compare_to(self, other: object) -> Optional[Dict[str, Set[str]]]:
+    def compare_to(self, other: object) -> Optional[dict[str, set[str]]]:
         """Compare two fingerprints and return the differences.
 
         :return: a dictionary that list the differences or None if the two
