@@ -6,6 +6,7 @@ import glob
 import hashlib
 import itertools
 import os
+import re
 import shutil
 import stat
 import sys
@@ -16,11 +17,13 @@ import e3
 import e3.error
 import e3.log
 import e3.os.fs
+from e3.collection.trie import Trie
 
 logger = e3.log.getLogger("fs")
 
 if TYPE_CHECKING:
-    from typing import Callable, Iterable, List, Optional, Sequence, Tuple, Union
+    from typing import Iterable, Optional
+    from collections.abc import Callable, Sequence
 
 
 class FSError(e3.error.E3Error):
@@ -63,7 +66,7 @@ def cp(
 
     if file_number == 0:
         # If there is no source files raise an error
-        raise FSError(origin="cp", message='can\'t find files matching "%s"' % source)
+        raise FSError(origin="cp", message=f'can\'t find files matching "{source}"')
     elif file_number > 1:
         # If we have more than one file to copy then check that target is a
         # directory
@@ -89,13 +92,13 @@ def cp(
         except Exception as e:
             logger.error(e, exc_info=True)
             raise FSError(
-                origin="cp", message="error occurred while copying %s" % f
-            ).with_traceback(sys.exc_info()[2])
+                origin="cp", message=f"error occurred while copying {f}"
+            ) from e
 
 
 def directory_content(
     path: str, include_root_dir: bool = False, unixpath: bool = False
-) -> List[str]:
+) -> list[str]:
     """Return the complete directory content (recusrsively).
 
     :param path: path for the which the content should be returned
@@ -125,9 +128,7 @@ def directory_content(
     return result
 
 
-def echo_to_file(
-    filename: str, content: Union[str, List[str]], append: bool = False
-) -> None:
+def echo_to_file(filename: str, content: str | list[str], append: bool = False) -> None:
     """Output content into a file.
 
     This function is useful when writing few content to a file for which we
@@ -155,7 +156,7 @@ def find(
     include_dirs: bool = False,
     include_files: bool = True,
     follow_symlinks: bool = False,
-) -> List[str]:
+) -> list[str]:
     """Find files or directory recursively.
 
     :param root: directory from which the research start
@@ -168,8 +169,8 @@ def find(
     :return: a list of files
     """
     result = []
-    for root, dirs, files in os.walk(root, followlinks=follow_symlinks):
-        root = root.replace("\\", "/")
+    for rt, dirs, files in os.walk(root, followlinks=follow_symlinks):
+        root = rt.replace("\\", "/")
         if include_files:
             for f in files:
                 if pattern is None or fnmatch.fnmatch(f, pattern):
@@ -229,8 +230,8 @@ def get_filetree_state(path: str, ignore_hidden: bool = True) -> str:
     return result.hexdigest()
 
 
-def ls(path: Union[str, List[str]], emit_log_record: bool = True) -> List[str]:
-    """List files.
+def ls(path: str | list[str], emit_log_record: bool = True) -> list[str]:
+    """list files.
 
     :param path: glob pattern or glob pattern list
     :param emit_log_record: if True, emit a log (debug) record
@@ -278,12 +279,10 @@ def mkdir(path: str, mode: int = 0o755, quiet: bool = False) -> None:
                 # existence and the call to makedirs
                 return
             logger.error(e)
-            raise FSError(
-                origin="mkdir", message="can't create %s" % path
-            ).with_traceback(sys.exc_info()[2])
+            raise FSError(origin="mkdir", message=f"can't create {path}") from e
 
 
-def mv(source: Union[str, List[str]], target: str) -> None:
+def mv(source: str | list[str], target: str) -> None:
     """Move files.
 
     :param source: a glob pattern
@@ -306,11 +305,14 @@ def mv(source: Union[str, List[str]], target: str) -> None:
             if hasattr(os.path, "samefile"):
                 try:
                     return os.path.samefile(src, dst)
-                except OSError:
+                except OSError:  # defensive code
+                    # This should never be raised as we called already os.stat
+                    # on both src and dst.
                     return False
-            return os.path.normcase(os.path.abspath(src)) == os.path.normcase(
-                os.path.abspath(dst)
-            )
+            else:  # defensive code (samefile supported on linux and windows)
+                return os.path.normcase(os.path.abspath(src)) == os.path.normcase(
+                    os.path.abspath(dst)
+                )
 
         def basename(path: str) -> str:
             sep = os.path.sep + (os.path.altsep or "")
@@ -335,10 +337,10 @@ def mv(source: Union[str, List[str]], target: str) -> None:
 
             real_dst = os.path.join(dst, basename(src))
             if os.path.exists(real_dst):
-                raise FSError("Destination path '%s' already exists" % real_dst)
+                raise FSError(f"Destination path '{real_dst}' already exists")
         try:
             os.rename(src, real_dst)
-        except OSError:
+        except OSError as err:
             if os.path.islink(src):
                 linkto = os.readlink(src)
                 os.symlink(linkto, real_dst)
@@ -346,8 +348,8 @@ def mv(source: Union[str, List[str]], target: str) -> None:
             elif os.path.isdir(src):
                 if destinsrc(src, dst):
                     raise FSError(
-                        "Cannot move a directory '%s' into itself" " '%s'." % (src, dst)
-                    )
+                        "Cannot move a directory '%s' into itself '%s'." % (src, dst)
+                    ) from err
                 shutil.copytree(src, real_dst, symlinks=True)
                 rm(src, recursive=True)
             else:
@@ -366,9 +368,7 @@ def mv(source: Union[str, List[str]], target: str) -> None:
         nb_files = len(file_list)
 
         if nb_files == 0:
-            raise FSError(
-                origin="mv", message='cannot find files matching "%s"' % source
-            )
+            raise FSError(origin="mv", message=f'cannot find files matching "{source}"')
         elif nb_files == 1:
             source = file_list[0]
             if os.path.isdir(source) and os.path.isdir(target):
@@ -377,7 +377,7 @@ def mv(source: Union[str, List[str]], target: str) -> None:
                 move_file(source, target)
         elif not os.path.isdir(target):
             # More than one file to move but the target is not a directory
-            raise FSError("mv", "%s should be a directory" % target)
+            raise FSError("mv", f"{target} should be a directory")
         else:
             for f in file_list:
                 f_dest = os.path.join(target, os.path.basename(f))
@@ -385,10 +385,10 @@ def mv(source: Union[str, List[str]], target: str) -> None:
                 move_file(f, f_dest)
     except Exception as e:
         logger.error(e)
-        raise FSError(origin="mv", message=str(e)).with_traceback(sys.exc_info()[2])
+        raise FSError(origin="mv", message=str(e)) from e
 
 
-def rm(path: Union[str, List[str]], recursive: bool = False, glob: bool = True) -> None:
+def rm(path: str | list[str], recursive: bool = False, glob: bool = True) -> None:
     """Remove files.
 
     :param path: a glob pattern, or a list of glob patterns
@@ -415,7 +415,7 @@ def rm(path: Union[str, List[str]], recursive: bool = False, glob: bool = True)
         else:
             file_list = set(path)
 
-    def onerror(func: Callable, error_path: str, exc_info: Tuple) -> None:
+    def onerror(func: Callable, error_path: str, exc_info: tuple) -> None:
         """When shutil.rmtree fail, try again to delete the file.
 
         :param func: function to call on error
@@ -483,17 +483,17 @@ def rm(path: Union[str, List[str]], recursive: bool = False, glob: bool = True)
         except Exception as e:  # defensive code
             logger.error(e)
             raise FSError(
-                origin="rm", message="error occurred while removing %s" % f
-            ).with_traceback(sys.exc_info()[2])
+                origin="rm", message=f"error occurred while removing {f}"
+            ) from e
 
 
-def splitall(path: str) -> Tuple[str, ...]:
+def splitall(path: str) -> tuple[str, ...]:
     """Split a path into a list of path components.
 
     :param path: path to split
     :return: a list of path components
     """
-    dirnames = []  # type: List[str]
+    dirnames = []  # type: list[str]
     while 1:
         head, tail = os.path.split(path)
         if head == path:
@@ -539,12 +539,12 @@ VCS_IGNORE_LIST = (
 def sync_tree(
     source: str,
     target: str,
-    ignore: Optional[Union[str, Sequence[str]]] = None,
-    file_list: Optional[List[str]] = None,
+    ignore: Optional[str | Sequence[str]] = None,
+    file_list: Optional[list[str]] = None,
     delete: bool = True,
     preserve_timestamps: bool = True,
     delete_ignore: bool = False,
-) -> Tuple[List[str], List[str]]:
+) -> tuple[list[str], list[str]]:
     """Synchronize the files and directories between two directories.
 
     :param source: the directory from where the files and directories
@@ -595,9 +595,29 @@ def sync_tree(
 
     # normalize ignore patterns
     if ignore is not None:
+        ignore = [ignore] if isinstance(ignore, str) else ignore
         norm_ignore_list = [fn.replace("\\", "/") for fn in ignore]
-        abs_ignore_patterns = [fn for fn in norm_ignore_list if fn.startswith("/")]
-        rel_ignore_patterns = [fn for fn in norm_ignore_list if not fn.startswith("/")]
+
+        ignore_path_suffixes = Trie(use_suffix=True, match_delimiter="/")
+        ignore_path_prefixes = Trie(match_delimiter="/")
+
+        ignore_base_regexp_list = []
+        ignore_base_regexp: Optional[re.Pattern[str]] = None
+
+        for pattern in norm_ignore_list:
+            pk = path_key(pattern)
+            if "/" not in pk:
+                # This is a regexp on the basename using fnmatch.
+                ignore_base_regexp_list.append(fnmatch.translate(pk))
+            elif pattern.startswith("/"):
+                # An absolute path
+                ignore_path_prefixes.add(pk)
+            else:
+                # A relative path
+                ignore_path_suffixes.add(pk)
+
+        if ignore_base_regexp_list:
+            ignore_base_regexp = re.compile("|".join(ignore_base_regexp_list))
 
     def is_in_ignore_list(p: str) -> bool:
         """Check if a file should be ignored.
@@ -612,20 +632,11 @@ def sync_tree(
         pk = path_key(p)
 
         return (
-            any(
-                f
-                for f in abs_ignore_patterns
-                if pk == path_key(f) or pk.startswith(path_key(f + "/"))
-            )
-            or any(
-                f
-                for f in rel_ignore_patterns
-                if pk[1:] == path_key(f) or p.endswith("/" + f)
-            )
-            or any(
-                f
-                for f in norm_ignore_list
-                if "/" not in f and fnmatch.fnmatch(os.path.basename(p), f)
+            ignore_path_prefixes.match(pk)
+            or ignore_path_suffixes.match(pk)
+            or (
+                ignore_base_regexp is not None
+                and bool(re.match(ignore_base_regexp, os.path.basename(pk)))
             )
         )
 
@@ -722,11 +733,11 @@ def sync_tree(
         if islink(src):  # windows: no cover
             mode = stat.S_IMODE(src.stat.st_mode)
             if hasattr(os, "lchmod"):
-                os.lchmod(dst.path, mode)
+                getattr(os, "lchmod")(dst.path, mode)  # noqa: B009
 
             if hasattr(os, "lchflags") and hasattr(src.stat, "st_flags"):
                 try:
-                    os.lchflags(dst.path, src.stat.st_flags)
+                    getattr(os, "lchflags")(dst.path, src.stat.st_flags)  # noqa: B009
                 except OSError as why:  # defensive code
                     import errno
 
@@ -746,7 +757,7 @@ def sync_tree(
                 os.chmod(dst.path, mode)
             if hasattr(os, "chflags") and hasattr(src.stat, "st_flags"):
                 try:
-                    os.chflags(dst.path, src.stat.st_flags)
+                    getattr(os, "chflags")(dst.path, src.stat.st_flags)  # noqa: B009
                 except OSError as why:  # defensive code
                     import errno
 
@@ -790,12 +801,12 @@ def sync_tree(
 
                 with open(src.path, "rb") as fsrc:
                     with open(dst.path, "wb") as fdst:
-                        shutil.copyfileobj(fsrc, fdst)  # type: ignore
+                        shutil.copyfileobj(fsrc, fdst)
             except OSError:
                 rm(dst.path, glob=False)
                 with open(src.path, "rb") as fsrc:
                     with open(dst.path, "wb") as fdst:
-                        shutil.copyfileobj(fsrc, fdst)  # type: ignore
+                        shutil.copyfileobj(fsrc, fdst)
             copystat(src, dst)
 
     def safe_mkdir(src: FileInfo, dst: FileInfo) -> None:
@@ -807,19 +818,31 @@ def sync_tree(
             rm(dst.path, glob=False)
 
         try:
-            if isdir(dst):
-                if dst.basename != src.basename:
-                    os.rename(
-                        dst.path, os.path.join(os.path.dirname(dst.path), src.basename)
-                    )
+            # Final dirname with right casing
+            if dst.basename != src.basename:
+                dest_dir = os.path.join(os.path.dirname(dst.path), src.basename)
             else:
-                os.makedirs(dst.path)
+                dest_dir = dst.path
+
+            if isdir(dst):
+                # For directories in case of non-matching casing just do a rename
+                # This ensure sync_tree is efficient in case content of the directory
+                # is similar between src and dst.
+                if dst.basename != src.basename:
+                    os.rename(dst.path, dest_dir)
+            else:
+                os.makedirs(dest_dir)
         except OSError:
             # in case of error to change parent directory
             # permissions. The permissions will be then
             # set correctly at the end of rsync.
             e3.os.fs.chmod("a+wx", os.path.dirname(dst.path))
-            os.makedirs(dst.path)
+
+            if isdir(dst):
+                if dst.basename != src.basename:
+                    os.rename(dst.path, dest_dir)
+            else:
+                os.makedirs(dest_dir)
 
     def walk(
         root_dir: str, target_root_dir: str, entry: Optional[FilesInfo] = None
@@ -923,11 +946,11 @@ def sync_tree(
     )
 
     if not os.path.exists(source):
-        raise FSError(origin="sync_tree", message="%s does not exist" % source)
+        raise FSError(origin="sync_tree", message=f"{source} does not exist")
 
     # Keep track of deleted and updated files
-    deleted_list: List[str] = []
-    updated_list: List[str] = []
+    deleted_list: list[str] = []
+    updated_list: list[str] = []
 
     for wf in walk(source_top, target_top):
         if wf.source.stat is None and wf.target.stat is not None:
