@@ -12,9 +12,12 @@ from e3.os.process import Run
 
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, List, Optional
+    from typing import Any, Optional
+    from collections.abc import Callable
     from e3.job.scheduler import Scheduler
 
+    NotifyEndType = Callable[[str], None]
+
 logger = e3.log.getLogger("job")
 
 
@@ -55,7 +58,7 @@ class Job(metaclass=abc.ABCMeta):
     lock = threading.RLock()
     index_counter = 0
 
-    def __init__(self, uid: str, data: Any, notify_end: Callable[[str], None]):
+    def __init__(self, uid: str, data: Any, notify_end: NotifyEndType):
         """Initialize worker.
 
         :param uid: unique work identifier
@@ -184,7 +187,7 @@ class EmptyJob(Job):
         uid: str,
         data: Any,
         notify_end: Callable[[str], None],
-        status: ReturnValue = ReturnValue.force_skip,
+        status: ReturnValue,
     ) -> None:
         """Initialize the EmptyJob.
 
@@ -254,7 +257,11 @@ class ProcessJob(Job, metaclass=abc.ABCMeta):
             return ReturnValue.notready
         else:
             try:
-                return ReturnValue(self.proc_handle.status)
+                if self.proc_handle.status is None:
+                    logger.exception("job %s returned None for status", self.uid)
+                    return ReturnValue.failure
+                else:
+                    return ReturnValue(self.proc_handle.status)
             except ValueError:
                 logger.exception(
                     "job %s returned an unknown status %s",
@@ -264,7 +271,7 @@ class ProcessJob(Job, metaclass=abc.ABCMeta):
                 return ReturnValue.failure
 
     @abc.abstractproperty
-    def cmdline(self) -> List[str]:
+    def cmdline(self) -> list[str]:
         """Return the command line of the process to be spawned.
 
         :return: the command line
