@@ -10,7 +10,8 @@ from e3.job import EmptyJob
 from e3.job.scheduler import DEFAULT_JOB_MAX_DURATION, Scheduler
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, FrozenSet, Optional, Set
+    from typing import Any, Optional
+    from collections.abc import Callable
     from e3.collection.dag import DAG
     from e3.fingerprint import Fingerprint
     from e3.job import Job, ProcessJob
@@ -27,16 +28,16 @@ class Walk:
           such fingerprint.
           (with the job corresponding to a given entry in the DAG of
           actions).
-    :vartype prev_fingerprints: Dict[str, Optional[Fingerprint]]
+    :vartype prev_fingerprints: dict[str, Optional[Fingerprint]]
     :ivar new_fingerprints: A dict of e3.fingerprint.Fingerprint objects,
         indexed by the corresponding job ID. This dictionary contains
         the fingerprints we compute each time we create a new job
         (with the job corresponding to a given entry in the DAG of
         actions).
-    :vartype new_fingerprints: Dict[str, Optional[Fingerprint]]
+    :vartype new_fingerprints: dict[str, Optional[Fingerprint]]
     :ivar job_status: A dictionary of job status (ReturnValue), indexed by
         job unique IDs.
-    :vartype job_status: Dict[str, ReturnValue]
+    :vartype job_status: dict[str, ReturnValue]
     :ivar scheduler: The scheduler used to schedule and execute all
         the actions.
     :vartype scheduler: e3.job.scheduler.Scheduler
@@ -48,11 +49,14 @@ class Walk:
         :param actions: DAG of actions to perform.
         """
         self.actions = actions
-        self.prev_fingerprints: Dict[str, Optional[Fingerprint]] = {}
-        self.new_fingerprints: Dict[str, Optional[Fingerprint]] = {}
-        self.job_status: Dict[str, ReturnValue] = {}
+        self.prev_fingerprints: dict[str, Optional[Fingerprint]] = {}
+        self.new_fingerprints: dict[str, Optional[Fingerprint]] = {}
+        self.job_status: dict[str, ReturnValue] = {}
+        self.queues: dict[str, int] = {}
+        self.tokens = 1
+        self.job_timeout = DEFAULT_JOB_MAX_DURATION
         self.set_scheduling_params()
-        self.failure_source: Dict[str, Set[str]] = {}
+        self.failure_source: dict[str, set[str]] = {}
 
         self.scheduler = Scheduler(
             job_provider=self.get_job,
@@ -78,9 +82,7 @@ class Walk:
         DEFAULT_JOB_MAX_DURATION. Child classes requiring different
         scheduling parameters should override this method.
         """
-        self.queues = None
-        self.tokens = 1
-        self.job_timeout = DEFAULT_JOB_MAX_DURATION
+        pass
 
     def compute_fingerprint(
         self, uid: str, data: Any, is_prediction: bool = False
@@ -161,6 +163,7 @@ class Walk:
               the given action;
             - when fingerprints are in use, execute the given action
               if the fingerprint has changed.
+
         However, child classes may want to override this method
         to implement alternative strategies.
 
@@ -187,7 +190,7 @@ class Walk:
         self,
         uid: str,
         data: Any,
-        predecessors: FrozenSet[str],
+        predecessors: frozenset[str],
         reason: Optional[str],
         notify_end: Callable[[str], None],
         status: ReturnValue = ReturnValue.failure,
@@ -210,7 +213,7 @@ class Walk:
         self,
         uid: str,
         data: Any,
-        predecessors: FrozenSet[str],
+        predecessors: frozenset[str],
         notify_end: Callable[[str], None],
     ) -> ProcessJob:
         """Create a ProcessJob.
@@ -236,7 +239,7 @@ class Walk:
         self,
         uid: str,
         data: Any,
-        predecessors: FrozenSet[str],
+        predecessors: frozenset[str],
         notify_end: Callable[[str], None],
     ) -> Job:
         """Return a Job.
@@ -265,7 +268,6 @@ class Walk:
             not in (
                 ReturnValue.success,
                 ReturnValue.skip,
-                ReturnValue.force_skip,
                 ReturnValue.unchanged,
             )
         ]
@@ -318,7 +320,6 @@ class Walk:
         # to skipping it).
         if job.status in (
             ReturnValue.success,
-            ReturnValue.force_skip,
             ReturnValue.skip,
             ReturnValue.unchanged,
         ):
@@ -328,7 +329,7 @@ class Walk:
         self.job_status[job.uid] = ReturnValue(job.status)
 
         if job.should_skip:
-            if job.status not in (ReturnValue.force_fail, ReturnValue.force_skip):
+            if job.status not in (ReturnValue.force_fail, ReturnValue.skip):
                 logging.info(
                     "[%-10s %-9s %4ds] %s",
                     job.queue_name,
