@@ -4,6 +4,7 @@ from __future__ import annotations
 from dataclasses import dataclass
 
 import logging
+import os
 import re
 import sys
 import time
@@ -11,7 +12,6 @@ import json
 from typing import TYPE_CHECKING, ClassVar
 
 from colorama import Fore, Style
-
 from tqdm import tqdm
 
 from e3.config import ConfigSection
@@ -21,15 +21,31 @@ if TYPE_CHECKING:
         Any,
         IO,
         Optional,
-        Iterator,
-        Sequence,
         TextIO,
-        Union,
-        List,
-        Tuple,
+        TypeVar,
         Mapping,
     )
+    from collections.abc import Iterator, Sequence
     from logging import _ExcInfoType
+    from argparse import ArgumentParser, _ArgumentGroup, Namespace
+
+    T = TypeVar("T")
+
+LEVELS = {
+    "DEBUG": logging.DEBUG,
+    "INFO": logging.INFO,
+    "WARNING": logging.WARNING,
+    "ERROR": logging.ERROR,
+    "CRITICAL": logging.CRITICAL,
+}
+
+# A list of python modules for which its dangerous to enable logging
+# at DEBUG level as it might leak some sensitive information. For the
+# following modules, by default logging will be limited to INFO when using
+# e3.main. This limitation can be bypassed by explicitely set back the
+# logging level to DEBUG: logging.getLogger(module).setLevel(logging.DEBUG)
+# after parsing the arguments.
+NO_DEBUG_LOGGING_MODULES = ["boto3", "botocore", "requests", "urllib3"]
 
 
 @dataclass
@@ -46,7 +62,7 @@ log_config = LogConfig.load()
 
 # Default output stream (sys.stdout by default, or a file descriptor if
 # activate() is called with a filename.
-default_output_stream: Union[TextIO, IO[str]] = sys.stdout
+default_output_stream: TextIO | IO[str] = sys.stdout
 
 # If sys.stdout is a terminal then enable "pretty" output for user
 # This includes progress bars and colors
@@ -68,7 +84,7 @@ class JSONFormatter(logging.Formatter):
     # standard attributes that will always be printed
     STD_ATTR = ["asctime", "levelname", "name", "message", "module", "exc_text"]
     # custom attributes
-    _extra_attr: List[str] = ["anod_uui"]
+    _extra_attr: list[str] = ["anod_uui"]
 
     def __init__(
         self,
@@ -108,7 +124,7 @@ class JSONFormatter(logging.Formatter):
 class E3LoggerAdapter(logging.LoggerAdapter):
     """LoggerAdapter to add custom keywords."""
 
-    def process(self, msg: Any, kwargs: Any) -> Tuple[Any, Any]:
+    def process(self, msg: Any, kwargs: Any) -> tuple[Any, Any]:
         """Allow to handle extra parameter.
 
         It is called by super method log. It is overwritten here because
@@ -184,7 +200,7 @@ class E3LoggerAdapter(logging.LoggerAdapter):
         )
 
 
-def progress_bar(it: Union[Iterator, Sequence], **kwargs: Any) -> tqdm:
+def progress_bar(it: Iterator[T] | Sequence[T], **kwargs: Any) -> Iterator[T]:
     """Create a tqdm progress bar.
 
     :param it: an interator
@@ -281,8 +297,8 @@ def add_log_handlers(
         file.
     """
     global default_output_stream
-    handler: Union[TqdmHandler, logging.StreamHandler, logging.FileHandler]
-    fmt: Union[logging.Formatter, JSONFormatter]
+    handler: TqdmHandler | logging.StreamHandler | logging.FileHandler
+    fmt: logging.Formatter | JSONFormatter
 
     if filename is None:
         if pretty_cli:  # all: no cover
@@ -306,6 +322,93 @@ def add_log_handlers(
     logging.getLogger("").addHandler(handler)
 
 
+def add_logging_argument_group(
+    argument_parser: ArgumentParser,
+    default_level: int = logging.WARNING,
+) -> _ArgumentGroup:
+    """Add an argument group with logging options to the argument parser.
+
+    To be used with `e3.log.activate_with_args`.
+
+    :param argument_parser: the parser in which the group will be created
+    :param default_level: the logging level that will be used by default
+    """
+    log_group = argument_parser.add_argument_group(title="logging arguments")
+    log_group.add_argument(
+        "-v",
+        "--verbose",
+        action="count",
+        default=0,
+        help="make the log output to the console more verbose",
+    )
+    log_group.add_argument(
+        "--log-file",
+        metavar="FILE",
+        default=None,
+        help="store all the logs into the specified file",
+    )
+    log_group.add_argument(
+        "--loglevel",
+        default=default_level,
+        help="set the console log level",
+        choices=LEVELS.keys(),
+    )
+    log_group.add_argument(
+        "--nocolor",
+        default=False,
+        action="store_true",
+        help="disable color and progress bars",
+    )
+    log_group.add_argument(
+        "--json-logs",
+        default="json-logs" in os.environ.get("E3_ENABLE_FEATURE", "").split(","),
+        action="store_true",
+        help="enable JSON formatted logs. They can be activated as well by"
+        " setting the env var E3_ENABLE_FEATURE=json-logs.",
+    )
+    log_group.add_argument(
+        "--console-logs",
+        metavar="LINE_PREFIX",
+        help="disable color, progress bars, and redirect as much as"
+        " possible to stdout, starting lines with the given prefix.",
+    )
+
+    return log_group
+
+
+def activate_with_args(args: Namespace, default_level: int = logging.WARNING) -> None:
+    """Activate the e3 log using argument parsed.
+
+    To be used with `e3.log.add_logging_argument_group`.
+
+    :param args: the result of parsing arguments
+    :param default_level: the logging level assumed by default
+    """
+    global console_logs
+    global pretty_cli
+
+    if args.verbose > 0:
+        level = default_level - 10 * args.verbose
+    else:
+        if args.loglevel in LEVELS:
+            level = LEVELS[args.loglevel]
+        else:
+            level = args.loglevel
+
+    if args.console_logs:
+        console_logs = args.console_logs
+
+    if args.nocolor:
+        pretty_cli = False
+
+    activate(
+        level=level,
+        filename=args.log_file,
+        json_format=args.json_logs,
+        e3_debug=level == logging.DEBUG,
+    )
+
+
 def activate(
     stream_format: str = log_config.stream_fmt,
     file_format: str = log_config.file_fmt,
@@ -327,6 +430,13 @@ def activate(
     # By default do not filter anything. What is effectively logged
     # will be defined by setting/unsetting handlers
     logging.getLogger("").setLevel(logging.DEBUG)
+
+    # Some thirdparties python modules might leak secrets. Ensure that we don't
+    # enable DEBUG level for them
+    if level < logging.INFO:
+        for module in NO_DEBUG_LOGGING_MODULES:
+            logging.getLogger(module).setLevel(logging.INFO)
+
     if console_logs:
         stream_format = f"{console_logs}: {file_format}"
 
