@@ -40,7 +40,8 @@ import e3.log
 from e3.env import Env
 
 if TYPE_CHECKING:
-    from typing import Optional, List
+    from types import FrameType
+    from typing import Optional, NoReturn
     from argparse import Namespace
 
 
@@ -54,6 +55,7 @@ class Main:
         self,
         name: Optional[str] = None,
         platform_args: bool = False,
+        default_x86_64_on_windows: bool = False,
         argument_parser: Optional[ArgumentParser] = None,
     ):
         """Initialize Main object.
@@ -64,6 +66,8 @@ class Main:
         :param argument_parser: the ArgumentParser to use for parsing
             command-line arguments (if not specified, an ArgumentParser will be
             created by Main)
+        :param default_x86_64_on_windows: set the default build platform to
+            x86_64-windows64 on Windows.
         """
         # On UNIX set a signal handler for SIGTERM that will raise SystemExit
         # This is to let an e3 application enough time to perform
@@ -74,60 +78,19 @@ class Main:
 
         if name is not None:
             self.name = name
-        else:
+        elif hasattr(main, "__file__"):
             self.name = os.path.splitext(os.path.basename(main.__file__))[0]
+        else:
+            self.name = "unknown"
 
         if argument_parser is None:
             argument_parser = ArgumentParser()
 
-        log_group = argument_parser.add_argument_group(title="Logging arguments")
-        log_group.add_argument(
-            "-v",
-            "--verbose",
-            action="count",
-            default=0,
-            help="make the log outputted on the console more verbose (this "
-            "sets the log level to DEBUG)",
-        )
-        log_group.add_argument(
-            "--log-file",
-            metavar="FILE",
-            default=None,
-            help="store all the logs into the specified file",
-        )
-        log_group.add_argument(
-            "--loglevel",
-            default=logging.INFO,
-            help="set the console log level",
-            choices={
-                "DEBUG": logging.DEBUG,
-                "INFO": logging.INFO,
-                "ERROR": logging.ERROR,
-                "CRITICAL": logging.CRITICAL,
-            },
-        )
-        log_group.add_argument(
-            "--nocolor",
-            default=False,
-            action="store_true",
-            help="disable color and progress bars",
-        )
-        log_group.add_argument(
-            "--json-logs",
-            default="json-logs" in os.environ.get("E3_ENABLE_FEATURE", "").split(","),
-            action="store_true",
-            help="enable JSON formatted logs. They can be activated as well by"
-            " setting the env var E3_ENABLE_FEATURE=json-logs.",
-        )
-        log_group.add_argument(
-            "--console-logs",
-            metavar="LINE_PREFIX",
-            help="disable color, progress bars, and redirect as much as"
-            " possible to stdout, starting lines with the given prefix.",
-        )
+        e3.log.add_logging_argument_group(argument_parser, default_level=logging.INFO)
 
         if platform_args:
-            plat_group = argument_parser.add_argument_group(title="Platform arguments")
+            plat_group = argument_parser.add_argument_group(title="platform arguments")
+
             plat_group.add_argument(
                 "--build",
                 default=None,  # to force autodetection
@@ -144,7 +107,7 @@ class Main:
                 default=None,  # to force autodetection
                 help="Set the target platform, target os version, "
                 "target machine, and target mode",
-                metavar="TARGET[,TARGET_VERSION[,TARGET_MACHINE[," "TARGET_MODE]]]",
+                metavar="TARGET[,TARGET_VERSION[,TARGET_MACHINE[,TARGET_MODE]]]",
             )
             # We add a default to a fake option as a way to encode
             # the fact that this parser supports the standard
@@ -157,12 +120,17 @@ class Main:
             # To avoid clashes with user-defined options, we use a dest
             # name that is improbable in practice.
             argument_parser.set_defaults(e3_main_platform_args_supported=True)
+            e = Env()
+            # we set the default build platform for windows if activated by
+            # the caller
+            if e.build.os.name == "windows" and default_x86_64_on_windows:
+                argument_parser.set_defaults(build="x86_64-windows64")
 
         self.args: Optional[Namespace] = None
         self.argument_parser = argument_parser
         self.__log_handlers_set = False
 
-        def sigterm_handler(sig, frame):  # unix-only
+        def sigterm_handler(sig: int, frame: FrameType) -> NoReturn:  # unix-only
             """Automatically convert SIGTERM to SystemExit exception.
 
             This is done to give enough time to an application killed by
@@ -178,7 +146,7 @@ class Main:
             signal.signal(signal.SIGTERM, sigterm_handler)
 
     def parse_args(
-        self, args: Optional[List[str]] = None, known_args_only: bool = False
+        self, args: Optional[list[str]] = None, known_args_only: bool = False
     ) -> None:
         """Parse options and set console logger.
 
@@ -192,25 +160,8 @@ class Main:
         else:
             self.args = self.argument_parser.parse_args(args)
 
-        if self.args.nocolor:
-            e3.log.pretty_cli = False
-
         if not self.__log_handlers_set:
-            # First set level of verbosity
-            if self.args.verbose:
-                level = logging.DEBUG
-            else:
-                level = self.args.loglevel
-
-            if self.args.console_logs:
-                e3.log.console_logs = self.args.console_logs
-
-            e3.log.activate(
-                level=level,
-                filename=self.args.log_file,
-                json_format=self.args.json_logs,
-                e3_debug=self.args.verbose > 1,
-            )
+            e3.log.activate_with_args(self.args, logging.INFO)
             self.__log_handlers_set = True
 
         # Export options to env
