@@ -22,7 +22,9 @@ from e3.error import E3Error
 from e3.fs import rm
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Deque, Dict, List, Optional, Tuple, Union
+    from types import TracebackType
+    from typing import Any, Deque, Optional
+    from collections.abc import Callable
     from requests.auth import AuthBase
     from requests.models import Response
 
@@ -41,7 +43,14 @@ def get_filename(content_disposition: str) -> Optional[str]:
 
 
 class HTTPError(E3Error):
-    pass
+    def __init__(self, msg: str, status: Optional[int] = None) -> None:
+        """Initialize an HTTPError exception.
+
+        :param msg: an error message
+        :param status: an optional HTTP status code
+        """
+        super().__init__(msg)
+        self.status = status
 
 
 class BaseURL:
@@ -57,7 +66,7 @@ class BaseURL:
         """
         self.url = url
 
-    def get_auth(self) -> Optional[Union[Tuple[str, str], AuthBase]]:
+    def get_auth(self) -> Optional[tuple[str, str] | AuthBase]:
         """Return auth requests parameter.
 
         :return: authentication associated with the url
@@ -73,7 +82,7 @@ class HTTPSession:
     CHUNK_SIZE = 1024 * 1024
     DEFAULT_TIMEOUT = (60, 60)
 
-    def __init__(self, base_urls: Optional[Union[List[str], List[BaseURL]]] = None):
+    def __init__(self, base_urls: Optional[list[str | BaseURL]] = None):
         """Initialize HTTP session.
 
         :param base_urls: list of urls used as prefix to subsequent requests.
@@ -81,7 +90,7 @@ class HTTPSession:
             during a request the next urls are used.
         """
         if base_urls:
-            self.base_urls: Union[Deque[BaseURL], Deque[None]] = deque(
+            self.base_urls: Deque[BaseURL] | Deque[None] = deque(
                 [k if isinstance(k, BaseURL) else BaseURL(k) for k in base_urls]
             )
         else:
@@ -93,7 +102,12 @@ class HTTPSession:
         self.session.__enter__()
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(
+        self,
+        exc_type: Optional[type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> None:
         self.session.__exit__(exc_type, exc_val, exc_tb)
 
     def set_max_retries(
@@ -128,7 +142,7 @@ class HTTPSession:
         self,
         method: str,
         url: str,
-        data_streams: Optional[Dict[str, Any]] = None,
+        data_streams: Optional[dict[str, Any]] = None,
         **kwargs: Any,
     ) -> Response:
         """Send a request.
@@ -141,13 +155,14 @@ class HTTPSession:
         For POST requests an additional parameter is supported: data_streams.
         data_streams is a dict associating a string key to either another
         string, a dict, a list or a file descriptor. String value are passed
-        without any modifications. Lists and dicts are automatically encoded
+        without any modifications. lists and dicts are automatically encoded
         in JSON. Finally file objects are streamed during the POST request
         (no complete read is done into memory to fetch file content). When
         using data_streams parameter, data parameter will be ignored and
         headers one modified.
         """
         error_msgs = []
+        last_status = None
 
         for base_url in list(self.base_urls):
             if self.last_base_url != base_url:
@@ -165,7 +180,7 @@ class HTTPSession:
             # Handle data_streams. After some tests it seems that we need to
             # redo an instance of the multipart encoder for each attempt.
             if data_streams is not None:
-                data: Dict[str, Any] = {}
+                data: dict[str, Any] = {}
                 for k, v in data_streams.items():
                     if hasattr(v, "seek"):
                         # This is a file. Assume that the key is the filename
@@ -188,7 +203,7 @@ class HTTPSession:
             # Compute final url
             if base_url is not None:
                 final_url = f"{base_url}/{url}"
-                message_prefix = "%s: " % base_url
+                message_prefix = f"{base_url}: "
             else:
                 final_url = url
                 message_prefix = ""
@@ -201,6 +216,7 @@ class HTTPSession:
                 response = self.session.request(method, final_url, **kwargs)
                 if response.status_code != 200:
                     error_msgs.append(f"{message_prefix}{response.text}")
+                    last_status = response.status_code
                     response.raise_for_status()
                 return response
             except (
@@ -214,7 +230,8 @@ class HTTPSession:
                 self.base_urls.append(problematic_url)  # type: ignore
 
         raise HTTPError(
-            "got request error (%d):\n%s" % (len(error_msgs), "\n".join(error_msgs))
+            "got request error (%d):\n%s" % (len(error_msgs), "\n".join(error_msgs)),
+            status=last_status,
         )
 
     def download_file(
@@ -223,6 +240,7 @@ class HTTPSession:
         dest: str,
         filename: Optional[str] = None,
         validate: Optional[Callable[[str], bool]] = None,
+        exception_on_error: bool = False,
     ) -> Optional[str]:
         """Download a file.
 
@@ -234,6 +252,8 @@ class HTTPSession:
         :param validate: function to call once the download is complete for
             detecting invalid / corrupted download. Takes the local path as
             parameter and returns a boolean.
+        :param exception_on_error: if True raise an exception in case download
+            fails instead of returning None.
         :return: the name of the file or None if there is an error
         """
         # When using stream=True, Requests cannot release the connection back
@@ -277,4 +297,6 @@ class HTTPSession:
             logger.debug(e)
             if path is not None:
                 rm(path)
+            if exception_on_error:
+                raise
         return None
