@@ -11,7 +11,8 @@ from e3.error import E3Error
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from typing import List, Optional
+    from typing import Optional
+    from collections.abc import Sequence
 
 logger = e3.log.getLogger("net.smtp")
 
@@ -20,10 +21,10 @@ system_sendmail_fallback = False
 
 def sendmail(
     from_email: str,
-    to_emails: List[str],
+    to_emails: list[str],
     mail_as_string: str,
-    smtp_servers: List[str],
-    max_size: int = 20,
+    smtp_servers: Sequence[str],
+    max_size: int = 40,
     message_id: Optional[str] = None,
 ) -> bool:
     """Send an email with stmplib.
@@ -56,11 +57,11 @@ def sendmail(
 
     def system_sendmail() -> bool:
         """Run the system sendmail."""
-        if system_sendmail_fallback:
+        if system_sendmail_fallback:  # all: no cover
             for sendmail_bin in (
                 "/usr/lib/sendmail",
                 "/usr/sbin/sendmail",
-            ):  # all: no cover
+            ):
                 if os.path.exists(sendmail_bin):
                     p = e3.os.process.Run(
                         [sendmail_bin] + to_emails,
@@ -84,23 +85,20 @@ def sendmail(
         except (OSError, smtplib.SMTPException) as e:
             logger.debug(e)
             logger.debug("cannot connect to smtp server %s", smtp_server)
-            continue
         else:
             try:
                 if not s.sendmail(from_email, to_emails, mail_as_string):
                     # sendmail returns an empty dictionary if the message
                     # was accepted for delivery to all addresses
                     break
-                continue
             except (OSError, smtplib.SMTPException) as e:
                 logger.debug(e)
                 logger.debug("smtp server error: %s", smtp_server)
-                continue
             finally:
                 try:
                     s.quit()
                     logger.debug("smtp quit")
-                except (OSError, smtplib.SMTPException):
+                except (OSError, smtplib.SMTPException):  # defensive code
                     # The message has already been delivered, ignore all errors
                     # when terminating the session.
                     pass
@@ -118,10 +116,10 @@ def sendmail(
 
 def send_message(
     from_email: str,
-    to_emails: List[str],
+    to_emails: list[str],
     subject: str,
     content: str,
-    smtp_servers: List[str],
+    smtp_servers: list[str],
 ) -> None:
     """Send an e-mail message.
 
