@@ -13,7 +13,7 @@ from e3.platform_db import get_knowledge_base
 
 
 if TYPE_CHECKING:
-    from typing import Optional, Tuple, Union
+    from typing import Any, Optional
 
 KNOWLEDGE_BASE = get_knowledge_base()
 
@@ -23,13 +23,12 @@ Uname = namedtuple(
     "Uname", ["system", "node", "release", "version", "machine", "processor"]
 )
 
+logger = e3.log.getLogger("os.platform")
+
 
 class SystemInfo:
     """Gather info about the system.
 
-    :cvar network_ifs: dictionary addressed by network interface name for which
-        each value is the result of netifaces.ifaddresses function on the given
-        interface
     :cvar uname: instance of Uname namedtuple containing the result of
         ``uname`` system call.
     :cvar core_number: integer containing the number of processor cores on the
@@ -37,7 +36,6 @@ class SystemInfo:
     :cvar nis_domain: host nis domain
     """
 
-    network_ifs = None
     uname = None
     core_number = 1
     nis_domain = None
@@ -47,23 +45,18 @@ class SystemInfo:
     _platform: Optional[str] = None
 
     # _os_version is a tuple: os version, kernel version
-    _os_version: Optional[Tuple[str, str]] = None
-
-    # When _is_virtual is True it means the host is a VM
-    _is_virtual: Optional[bool] = None
+    _os_version: Optional[tuple[str, str]] = None
 
     # _hostname is a tuble: hostname, domain. Joining with a dot hostname and domain
     # represent the FQDN
-    _hostname: Optional[Tuple[str, str]] = None
+    _hostname: Optional[tuple[str, str]] = None
 
     @classmethod
     def reset_cache(cls) -> None:
         """Reset SystemInfo cache."""
-        cls.network_ifs = None
         cls.uname = None
         cls.ld_info = None
         cls.core_number = 1
-        cls._is_virtual = None
         cls._platform = None
         cls._os_version = None
         cls._hostname = None
@@ -93,20 +86,6 @@ class SystemInfo:
                 "version": ld.version(),
             }
 
-        # Fetch network interfaces
-        try:
-            from netifaces import interfaces, ifaddresses, address_families
-
-            # use string for address families instead of integers which are
-            # system dependents
-            cls.network_ifs = {
-                itf: {address_families[k]: v for k, v in ifaddresses(itf).items()}
-                for itf in interfaces()
-            }
-        except Exception:  # defensive code
-            e3.log.debug("cannot get network info", exc_info=True)
-            cls.network_ifs = None
-
         # Fetch core numbers. Note that the methods does not work
         # on AIX platform but we usually override manually that
         # setting anyway.
@@ -116,13 +95,13 @@ class SystemInfo:
 
             cls.core_number = multiprocessing.cpu_count()
         except Exception:  # defensive code
-            e3.log.debug("multiprocessing error", exc_info=True)
+            logger.exception("multiprocessing error")
             try:
                 import psutil
 
                 cls.core_number = psutil.cpu_count()
             except Exception:
-                e3.log.debug("psutil error", exc_info=True)
+                logger.exception("psutil error")
 
         cls.nis_domain = UNKNOWN
 
@@ -130,7 +109,6 @@ class SystemInfo:
             try:
                 import nis
             except ImportError:  # defensive code
-                e3.log.debug("cannot import nis", exc_info=True)
                 nis = None  # type: ignore
 
             if nis is not None:
@@ -139,7 +117,9 @@ class SystemInfo:
                     if not cls.nis_domain:  # defensive code
                         cls.nis_domain = UNKNOWN
                 except nis.error:  # defensive code
-                    e3.log.debug("nis error", exc_info=True)
+                    # this is not a problem anymore, we do not want
+                    # to log it either, so we just ignore it.
+                    pass
 
     @classmethod
     def platform(cls) -> str:
@@ -178,7 +158,7 @@ class SystemInfo:
         return cls._platform
 
     @classmethod
-    def os_version(cls) -> Tuple[str, str]:
+    def os_version(cls) -> tuple[str, str]:
         """Compute OS version information.
 
         :return: a tuple containing os version and kernel version
@@ -244,9 +224,7 @@ class SystemInfo:
                     ("wReserved", ctypes.c_byte),
                 ]
 
-            def get_os_version() -> Union[
-                Tuple[None, None, None], Tuple[float, int, bool]
-            ]:
+            def get_os_version() -> tuple[None, None, None] | tuple[float, int, bool]:
                 """Return the real Windows kernel version.
 
                 On recent version, the kernel version returned by the
@@ -288,8 +266,10 @@ class SystemInfo:
                     if is_server:
                         if build_number < 17763:
                             version = "2016"
-                        else:
+                        elif build_number < 20348:
                             version = "2019"
+                        else:
+                            version = "2022"
                     else:
                         version = "10"
 
@@ -297,48 +277,7 @@ class SystemInfo:
         return version, kernel_version
 
     @classmethod
-    def is_virtual(cls) -> bool:
-        """Check if current machine is virtual or not.
-
-        :return: True if the machine is a virtual machine (Solaris zone,
-            VmWare)
-        """
-        if cls._is_virtual is not None:
-            return cls._is_virtual
-
-        if cls.uname is None:
-            cls.fetch_system_data()
-
-        if TYPE_CHECKING:
-            assert cls.uname is not None
-
-        result = False
-
-        if (
-            cls.uname.system == "SunOS" and cls.uname.version == "Generic_Virtual"
-        ):  # solaris-only
-            result = True
-        else:
-            if cls.network_ifs is not None:
-                for interface in list(cls.network_ifs.values()):
-                    for family in ("AF_LINK", "AF_PACKET"):
-                        if family in interface:
-                            for el in interface[family]:
-                                addr = el["addr"].lower()
-                                if addr.startswith("00:0c:29") or addr.startswith(
-                                    "00:50:56"
-                                ):
-                                    result = True
-                                    break
-                        if result:
-                            break
-                    if result:
-                        break
-        cls._is_virtual = result
-        return result
-
-    @classmethod
-    def hostname(cls) -> Tuple[str, str]:
+    def hostname(cls) -> tuple[str, str]:
         """Get hostname and associated domain.
 
         :return: a tuple (hostname, domain)
@@ -379,7 +318,7 @@ class CPU(namedtuple("CPU", ["name", "bits", "endian", "cores"])):
 
     __slots__ = ()
 
-    def as_dict(self):
+    def as_dict(self) -> dict[str, Any]:
         return self._asdict()
 
     @classmethod
@@ -432,7 +371,7 @@ class OS(
 
     __slots__ = ()
 
-    def as_dict(self):
+    def as_dict(self) -> dict[str, Any]:
         return self._asdict()
 
     @classmethod
