@@ -20,7 +20,7 @@ from ctypes.wintypes import (
     USHORT,
 )
 from datetime import datetime
-from typing import List, Optional
+from typing import Optional
 
 from e3.error import E3Error
 
@@ -28,7 +28,7 @@ NTSTATUS = LONG
 
 
 class FileAttribute(Structure):
-    """List of file attributes constants."""
+    """list of file attributes constants."""
 
     READONLY = 0x01
     HIDDEN = 0x02
@@ -50,7 +50,7 @@ class FileAttribute(Structure):
 
     _fields_ = [("attr", ULONG)]
 
-    def __str__(self):
+    def __str__(self) -> str:
         result = []
         for k in FileAttribute.__dict__:
             if not k.startswith("_") and k.isupper():
@@ -166,20 +166,20 @@ class FileTime(Structure):
 
     _fields_ = [("filetime", LARGE_INTEGER)]
 
-    def __init__(self, t):
+    def __init__(self, t: datetime) -> None:
         timestamp = (t - datetime(1970, 1, 1)).total_seconds()
         timestamp = (int(timestamp) + 11644473600) * 10000000
         Structure.__init__(self, timestamp)
 
     @property
-    def as_datetime(self):
+    def as_datetime(self) -> datetime:
         try:
             return datetime.fromtimestamp(self.filetime // 10000000 - 11644473600)
         except ValueError as err:  # defensive code
             # Add some information to ease debugging
-            raise ValueError(f"filetime '{self.filetime}' failed with {err}")
+            raise ValueError(f"filetime '{self.filetime}' failed with {err}") from err
 
-    def __str__(self):
+    def __str__(self) -> str:
         try:
             return str(time.ctime(self.filetime // 10000000 - 11644473600))
         except ValueError:  # defensive code
@@ -201,7 +201,7 @@ class FileInfo:
         class_id = 6
 
     class Rename(Structure):
-        _fields_ = []  # type: List
+        _fields_ = []  # type: list
         class_id = 10
 
     class Basic(Structure):
@@ -219,14 +219,22 @@ class FileInfo:
             self.file_attributes = FileAttribute(0)
 
         def __str__(self) -> str:
-            result = "creation_time:    %s\n" % self.creation_time
-            result += "last_access_time: %s\n" % self.last_access_time
-            result += "last_write_time:  %s\n" % self.last_write_time
-            result += "change_time:      %s\n" % self.change_time
-            result += "file_attributes:  %s" % self.file_attributes
+            result = f"creation_time:    {self.creation_time}\n"
+            result += f"last_access_time: {self.last_access_time}\n"
+            result += f"last_write_time:  {self.last_write_time}\n"
+            result += f"change_time:      {self.change_time}\n"
+            result += f"file_attributes:  {self.file_attributes}"
             return result
 
 
+class ObjectInfo:
+    """Declaration of structures returned by QueryObjectInformation."""
+
+    class Name(Structure):
+        _fields_: list = []
+        class_id = 1
+
+
 class ProcessInfo:
     """Declaration of structure returned by QueryInformationProcess."""
 
@@ -290,84 +298,95 @@ class NT:
     QueryInformationProcess = None
     WaitForMultipleObjects = None
     OpenProcess = None
+    QueryObject = None
 
     @classmethod
-    def init_api(cls):
-        kernel32 = ctypes.windll.kernel32
-        ntdll = ctypes.windll.ntdll
+    def init_api(cls) -> None:
+        if sys.platform == "win32":
+            kernel32 = ctypes.windll.kernel32
+            ntdll = ctypes.windll.ntdll
 
-        cls.GetVolumePathName = kernel32.GetVolumePathNameW
-        cls.GetVolumePathName.restype = BOOL
-        cls.GetVolumePathName.argtypes = [c_wchar_p, c_wchar_p, DWORD]
-        cls.Sleep = kernel32.Sleep
-        cls.Sleep.argtypes = [DWORD]
+            cls.GetVolumePathName = kernel32.GetVolumePathNameW
+            cls.GetVolumePathName.restype = BOOL
+            cls.GetVolumePathName.argtypes = [c_wchar_p, c_wchar_p, DWORD]
+            cls.Sleep = kernel32.Sleep
+            cls.Sleep.argtypes = [DWORD]
 
-        cls.SetInformationFile = ntdll.NtSetInformationFile
-        cls.SetInformationFile.restype = NTSTATUS
-        cls.SetInformationFile.argtypes = [
-            HANDLE,
-            POINTER(IOStatusBlock),
-            LPVOID,
-            ULONG,
-            INT,
-        ]
-        cls.QueryInformationFile = ntdll.NtQueryInformationFile
-        cls.QueryInformationFile.restype = NTSTATUS
-        cls.QueryInformationFile.argtypes = [
-            HANDLE,
-            POINTER(IOStatusBlock),
-            LPVOID,
-            ULONG,
-            INT,
-        ]
-        cls.QueryAttributesFile = ntdll.NtQueryAttributesFile
-        cls.QueryAttributesFile.restype = NTSTATUS
-        cls.QueryAttributesFile.argtypes = [
-            POINTER(ObjectAttributes),
-            POINTER(FileInfo.Basic),
-        ]
-        cls.OpenFile = ntdll.NtOpenFile
-        cls.OpenFile.restype = NTSTATUS
-        cls.OpenFile.argtypes = [
-            POINTER(HANDLE),
-            DWORD,
-            POINTER(ObjectAttributes),
-            POINTER(IOStatusBlock),
-            ULONG,
-            ULONG,
-        ]
-        cls.QueryDirectoryFile = ntdll.NtQueryDirectoryFile
-        cls.QueryDirectoryFile.restype = NTSTATUS
-        cls.QueryDirectoryFile.argtypes = [
-            HANDLE,
-            HANDLE,
-            LPVOID,
-            LPVOID,
-            POINTER(IOStatusBlock),
-            LPVOID,
-            ULONG,
-            INT,
-            BOOLEAN,
-            POINTER(UnicodeString),
-            BOOLEAN,
-        ]
-        cls.Close = ntdll.NtClose
-        cls.Close.argtypes = [HANDLE]
-        cls.QueryInformationProcess = ntdll.NtQueryInformationProcess
-        cls.QueryInformationProcess.restype = NTSTATUS
-        cls.QueryInformationProcess.argtypes = [
-            HANDLE,
-            INT,
-            POINTER(ProcessInfo.Basic),
-            ULONG,
-            LPVOID,
-        ]
-        cls.WaitForMultipleObjects = kernel32.WaitForMultipleObjects
-        cls.WaitForMultipleObjects.restype = DWORD
-        cls.WaitForMultipleObjects.argtypes = [DWORD, POINTER(HANDLE), BOOLEAN, DWORD]
-        cls.OpenProcess = kernel32.OpenProcess
-        cls.OpenProcess.restype = HANDLE
-        cls.OpenProcess.argtypes = [DWORD, BOOL, DWORD]
+            cls.SetInformationFile = ntdll.NtSetInformationFile
+            cls.SetInformationFile.restype = NTSTATUS
+            cls.SetInformationFile.argtypes = [
+                HANDLE,
+                POINTER(IOStatusBlock),
+                LPVOID,
+                ULONG,
+                INT,
+            ]
+            cls.QueryInformationFile = ntdll.NtQueryInformationFile
+            cls.QueryInformationFile.restype = NTSTATUS
+            cls.QueryInformationFile.argtypes = [
+                HANDLE,
+                POINTER(IOStatusBlock),
+                LPVOID,
+                ULONG,
+                INT,
+            ]
+            cls.QueryObject = ntdll.NtQueryObject
+            cls.QueryObject.restype = NTSTATUS
+            cls.QueryObject.argtypes = [HANDLE, INT, LPVOID, ULONG, LPVOID]
+
+            cls.QueryAttributesFile = ntdll.NtQueryAttributesFile
+            cls.QueryAttributesFile.restype = NTSTATUS
+            cls.QueryAttributesFile.argtypes = [
+                POINTER(ObjectAttributes),
+                POINTER(FileInfo.Basic),
+            ]
+            cls.OpenFile = ntdll.NtOpenFile
+            cls.OpenFile.restype = NTSTATUS
+            cls.OpenFile.argtypes = [
+                POINTER(HANDLE),
+                DWORD,
+                POINTER(ObjectAttributes),
+                POINTER(IOStatusBlock),
+                ULONG,
+                ULONG,
+            ]
+            cls.QueryDirectoryFile = ntdll.NtQueryDirectoryFile
+            cls.QueryDirectoryFile.restype = NTSTATUS
+            cls.QueryDirectoryFile.argtypes = [
+                HANDLE,
+                HANDLE,
+                LPVOID,
+                LPVOID,
+                POINTER(IOStatusBlock),
+                LPVOID,
+                ULONG,
+                INT,
+                BOOLEAN,
+                POINTER(UnicodeString),
+                BOOLEAN,
+            ]
+            cls.Close = ntdll.NtClose
+            cls.Close.argtypes = [HANDLE]
+            cls.QueryInformationProcess = ntdll.NtQueryInformationProcess
+            cls.QueryInformationProcess.restype = NTSTATUS
+            cls.QueryInformationProcess.argtypes = [
+                HANDLE,
+                INT,
+                POINTER(ProcessInfo.Basic),
+                ULONG,
+                LPVOID,
+            ]
+            cls.WaitForMultipleObjects = kernel32.WaitForMultipleObjects
+            cls.WaitForMultipleObjects.restype = DWORD
+            cls.WaitForMultipleObjects.argtypes = [
+                DWORD,
+                POINTER(HANDLE),
+                BOOLEAN,
+                DWORD,
+            ]
+            cls.OpenProcess = kernel32.OpenProcess
+            cls.OpenProcess.restype = HANDLE
+            cls.OpenProcess.argtypes = [DWORD, BOOL, DWORD]
 
 
 if sys.platform == "win32":
@@ -378,10 +397,11 @@ class NTException(E3Error):
     def __init__(self, status: int, message: str, origin: Optional[str] = None):
         self.status = status
         if self.status < 0:
-            self.status += 2 ** 32
+            self.status += 2**32
         E3Error.__init__(self, message, origin=origin)
 
     def __str__(self):
         return E3Error.__str__(self) + "(status={:X} '{}')".format(
-            self.status, Status.msgs.get(self.status, "unknown"),
+            self.status,
+            Status.msgs.get(self.status, "unknown"),
         )
