@@ -8,7 +8,7 @@ import e3.os.platform
 from e3.platform_db import get_knowledge_base
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, Optional
+    from typing import Any, Optional
 
 KNOWLEDGE_BASE = get_knowledge_base()
 
@@ -109,6 +109,7 @@ class Platform(
             is_default = platform_name == default_platform
 
         # Fill other attributes
+        assert platform_name is not None
         pi = KNOWLEDGE_BASE.platform_info[platform_name]
         cpu = e3.os.platform.CPU.get(pi["cpu"], pi.get("endian", None), is_host)
         os = e3.os.platform.OS.get(pi["os"], is_host, version=version, mode=mode)
@@ -131,23 +132,12 @@ class Platform(
             is_default,
         )
 
-    @property
-    def is_virtual(self) -> bool:
-        """Check if we are on a virtual system.
-
-        :return: True if the system represented by Arch is a virtual machine
-        """
-        if not self.is_host:
-            return False
-        return self.system_info.is_virtual()
-
-    def to_dict(self) -> Dict[str, Any]:
+    def to_dict(self) -> dict[str, Any]:
         """Export os and cpu variables as os_{var} and cpu_{var}.
 
         :return: a dictionary representing the current Arch instance
         """
         str_dict = self._asdict()
-        str_dict["is_virtual"] = self.is_virtual
 
         for key, var in self.os.as_dict().items():
             str_dict["os_" + key] = var
@@ -164,7 +154,6 @@ class Platform(
             "machine:  %(machine)s\n"
             "is_hie:   %(is_hie)s\n"
             "is_host:  %(is_host)s\n"
-            "is_virtual: %(is_virtual)s\n"
             "triplet:  %(triplet)s\n"
             "domain:   %(domain)s\n"
             "OS\n"
