@@ -5,8 +5,12 @@ knowledge base is very often read and the cost of parsing the data can
 be significant in some context.
 """
 from __future__ import annotations
+from typing import TYPE_CHECKING
 
-CPU_INFO = {
+if TYPE_CHECKING:
+    from e3.platform_db import PlatformDBEntry
+
+CPU_INFO: PlatformDBEntry = {
     "aarch64": {"endian": "little", "bits": 64},
     "arm": {"endian": "little", "bits": 32},
     "avr": {"endian": "little", "bits": 16},
@@ -18,7 +22,7 @@ CPU_INFO = {
     "x86_64": {"endian": "little", "bits": 64},
 }
 
-OS_INFO = {
+OS_INFO: PlatformDBEntry = {
     "aix": {"is_bareboard": False, "version": "5.2", "exeext": "", "dllext": ".so"},
     "darwin": {
         "is_bareboard": False,
@@ -72,7 +76,7 @@ OS_INFO = {
     "none": {"is_bareboard": True, "version": "unknown", "exeext": "", "dllext": ""},
 }
 
-PLATFORM_INFO = {
+PLATFORM_INFO: PlatformDBEntry = {
     "aarch64-elf": {"cpu": "aarch64", "os": "none", "is_hie": True},
     "aarch64-ios": {"cpu": "aarch64", "os": "ios", "is_hie": False},
     "aarch64-linux": {"cpu": "aarch64", "os": "linux", "is_hie": False},
@@ -109,7 +113,7 @@ PLATFORM_INFO = {
     "x86_64-windows64": {"cpu": "x86_64", "os": "windows", "is_hie": False},
 }
 
-BUILD_TARGETS = {
+BUILD_TARGETS: PlatformDBEntry = {
     "aarch64-elf": {"name": "aarch64-elf"},
     "aarch64-ios": {"name": "aarch64-apple-darwin"},
     "aarch64-linux": {"name": "aarch64-linux-gnu"},
@@ -150,8 +154,9 @@ BUILD_TARGETS = {
 # by a different host. IMPORTANT: Systems that can be only used as target in
 # cross context should not be added to that table.
 
-HOST_GUESS = {
+HOST_GUESS: PlatformDBEntry = {
     # platform : OS (uname[0]), machine (uname[1]), proc (uname[4 or 5])
+    "aarch64-linux": {"os": "Linux", "cpu": "aarch64"},
     "ppc-aix": {"os": "AIX", "cpu": None},
     "x86_64-darwin": {"os": "Darwin", "cpu": "i386"},
     "x86-freebsd": {"os": "FreeBSD", "cpu": "i386"},
