@@ -7,7 +7,7 @@ from e3.net.http import HTTPSession
 from e3.store.backends.base import ResourceInfo, Store, StoreError
 
 if TYPE_CHECKING:
-    from typing import Dict, Optional
+    from typing import Optional
 
 logger = e3.log.getLogger("store.httpsimplestore")
 
@@ -21,7 +21,7 @@ class HTTPSimpleStoreResourceInfo(ResourceInfo):
         resource_sha = e3.hash.sha1(resource_path)
         if resource_sha != self.sha:
             logger.critical(
-                "wrong sha for resource %s " "expecting %s got %s",
+                "wrong sha for resource %s expecting %s got %s",
                 resource_path,
                 self.sha,
                 resource_sha,
@@ -37,11 +37,12 @@ class HTTPSimpleStoreResourceInfo(ResourceInfo):
 
 class HTTPSimpleStore(Store):
     def get_resource_metadata(
-        self, query: Dict[str, str]
+        self, query: dict[str, str]
     ) -> HTTPSimpleStoreResourceInfo:
         """Return resource metadata directly computed from the query.
 
         There is no remote server involved here.
+
         :param query: a dict containing two keys 'sha' and 'url'. sha is the
             sha1sum of the resource and url is the remote url
         """
