@@ -35,16 +35,14 @@ from e3.vcs import VCSError
 if TYPE_CHECKING:
     from typing import (
         Any,
-        Dict,
         Final,
-        Iterator,
         IO,
         List,
         Literal,
         Optional,
         TextIO,
-        Union,
     )
+    from collections.abc import Iterator
     from e3.os.process import Run, STDOUT_VALUE, DEVNULL_VALUE, PIPE_VALUE
 
     Git_Cmd = List[Optional[str]]
@@ -79,7 +77,7 @@ class GitRepository:
     """
 
     git: Optional[str] = None
-    log_stream: Union[TextIO, IO[str]] = sys.stdout
+    log_stream: TextIO | IO[str] = sys.stdout
 
     def __init__(self, working_tree: str):
         """Initialize a GitRepository object.
@@ -111,10 +109,14 @@ class GitRepository:
     def git_cmd(
         self,
         cmd: Git_Cmd,
-        output: Union[
-            STDOUT_VALUE, DEVNULL_VALUE, PIPE_VALUE, GIT_LOG_STREAM_VALUE, str, IO, None
-        ] = GIT_LOG_STREAM,
-        **kwargs: Any
+        output: STDOUT_VALUE
+        | DEVNULL_VALUE
+        | PIPE_VALUE
+        | GIT_LOG_STREAM_VALUE
+        | str
+        | IO
+        | None = GIT_LOG_STREAM,
+        **kwargs: Any,
     ) -> Run:
         """Run a git command.
 
@@ -280,7 +282,7 @@ class GitRepository:
 
     def parse_log(
         self, stream: IO[str], max_diff_size: int = 0
-    ) -> Iterator[Dict[str, Union[str, dict]]]:
+    ) -> Iterator[dict[str, str | dict]]:
         """Parse a log stream generated with `write_log`.
 
         :param stream: stream of text to read
