@@ -1,6 +1,7 @@
 """High-Level interface to SVN repository.
 
 Example::
+
     svn = SVNRepository(working_copy='/tmp/working_dir')
     svn.update(url='file:///path_to_local_repo', revision=1,
                force_and_clean=True)
@@ -22,7 +23,7 @@ from e3.fs import mkdir, rm
 from e3.vcs import VCSError
 
 if TYPE_CHECKING:
-    from typing import Any, IO, List, Optional, TextIO, Tuple, Union
+    from typing import Any, IO, List, Optional, TextIO
 
     SVNCmd = List[Optional[str]]
 
@@ -43,7 +44,7 @@ class SVNRepository:
     """
 
     svn_bin = None
-    log_stream: Union[TextIO, IO[str]] = sys.stdout
+    log_stream: TextIO | IO[str] = sys.stdout
 
     def __init__(self, working_copy: str):
         """Initialize a SVNRepository object.
@@ -105,7 +106,7 @@ class SVNRepository:
 
         if p.status != 0:
             raise SVNError(
-                "cannot create svn repository in %s" % repo_path, origin="create"
+                f"cannot create svn repository in {repo_path}", origin="create"
             )
         if initial_content_path is not None:
             p = e3.os.process.Run(
@@ -115,14 +116,14 @@ class SVNRepository:
                     initial_content_path,
                     cls.local_url(repo_path),
                     "-m",
-                    "Initial import from %s" % initial_content_path,
+                    f"Initial import from {initial_content_path}",
                 ],
                 output=cls.log_stream,
             )
             if p.status != 0:
                 raise SVNError(
-                    "cannot perform initial import of %s into %s"
-                    % (initial_content_path, repo_path),
+                    f"cannot perform initial import of {initial_content_path}"
+                    f" into {repo_path}",
                     origin="create",
                 )
         return cls.local_url(repo_path)
@@ -132,6 +133,7 @@ class SVNRepository:
 
         Add the non-interactive option to all command (accepted on all SVN.
         subcommands from version 1.5).
+
         :param cmd: the command line as a list of string, all None entries will
             be discarded
         :param kwargs: additional parameters to provide to e3.os.process.Run
@@ -147,6 +149,12 @@ class SVNRepository:
         if "output" not in kwargs:
             kwargs["output"] = self.log_stream
 
+        if "env" not in kwargs:
+            kwargs["env"] = {"LC_ALL": "C"}
+            kwargs["ignore_environ"] = False
+        else:
+            kwargs["env"]["LC_ALL"] = "C"
+
         p_cmd = [arg for arg in cmd if arg is not None]
         p_cmd.insert(0, self.__class__.svn_bin)
         p_cmd.append("--non-interactive")
@@ -169,12 +177,10 @@ class SVNRepository:
         :raise: SVNError
         """
         info = self.svn_cmd(["info"], output=PIPE).out
-        m = re.search(fr"^{item}: *(.*)\n", info, flags=re.M)
+        m = re.search(rf"^{item}: *(.*)\n", info, flags=re.M)
         if m is None:
             logger.debug("svn info result:\n%s", info)
-            raise SVNError(
-                "Cannot fetch item %s from svn_info" % item, origin="get_info"
-            )
+            raise SVNError(f"Cannot fetch item {item} from svn_info", origin="get_info")
         return m.group(1).strip()
 
     @property
@@ -193,9 +199,9 @@ class SVNRepository:
         """
         try:
             return self.get_info("Last Changed Rev")
-        except Exception:
+        except Exception as err:
             logger.exception("Cannot fetch last changed rev")
-            raise SVNError("Cannot fetch last changed rev", "svn_cmd")
+            raise SVNError("Cannot fetch last changed rev", "svn_cmd") from err
 
     def update(
         self,
@@ -219,7 +225,7 @@ class SVNRepository:
         :raise: SVNError
         """
 
-        def is_clean_svn_dir(dir_path: str) -> Tuple[bool, bool]:
+        def is_clean_svn_dir(dir_path: str) -> tuple[bool, bool]:
             """Return a tuple (True if dir is SVN directory, True if clean)."""
             if os.path.exists(os.path.join(dir_path, ".svn")):
                 try:
@@ -253,7 +259,8 @@ class SVNRepository:
         if os.path.exists(self.working_copy):
             if not is_empty_dir(self.working_copy) and not force_and_clean:
                 raise SVNError(
-                    f"not empty {self.working_copy} url {url}", origin="update",
+                    f"not empty {self.working_copy} url {url}",
+                    origin="update",
                 )
             if is_svn_dir and not url:
                 url = self.url
