@@ -5,7 +5,6 @@ from __future__ import annotations
 
 import os
 import re
-import sys
 from collections import OrderedDict
 from typing import TYPE_CHECKING
 
@@ -19,7 +18,7 @@ from e3.text import format_with_dict
 if TYPE_CHECKING:
     # Conditonal imports do not work with mypy, unconditionaly use yaml.Loader
     # for type checking
-    from typing import Any, IO, List, Set, Tuple, Union
+    from typing import Any, IO
     from yaml import Loader
 else:
     try:
@@ -79,7 +78,7 @@ class OrderedDictYAMLLoader(Loader):
             raise yaml.constructor.ConstructorError(
                 context=None,
                 context_mark=None,
-                problem="expected a mapping node, but found %s" % node.id,
+                problem=f"expected a mapping node, but found {node.id}",
                 problem_mark=node.start_mark,
             )
 
@@ -92,15 +91,15 @@ class OrderedDictYAMLLoader(Loader):
                 raise yaml.constructor.ConstructorError(
                     context="while constructing a mapping",
                     context_mark=node.start_mark,
-                    problem="found unacceptable key (%s)" % exc,
+                    problem=f"found unacceptable key ({exc})",
                     problem_mark=key_node.start_mark,
-                )
+                ) from exc
             value = self.construct_object(value_node, deep=deep)
             if key in mapping:
                 raise yaml.constructor.ConstructorError(
                     context="while constructing a mapping",
                     context_mark=node.start_mark,
-                    problem="found duplicate key (%s)" % key,
+                    problem=f"found duplicate key ({key})",
                     problem_mark=key_node.start_mark,
                 )
             mapping[key] = value
@@ -159,7 +158,7 @@ class CaseParser:
         # This contains the list of keys that have been updated. This
         # allow us to remove the keys part of initial_config that are
         # not modified.
-        self.keys: Set[str] = set()
+        self.keys: set[str] = set()
 
     def __parse_case(self, case_key: str, data: dict) -> Any:
         """Parse a case statement.
@@ -173,7 +172,7 @@ class CaseParser:
         key_val = str(self.__state[key])
 
         result = next(
-            ((key_val, k, data[k]) for k in data if re.match("^%s$" % k, key_val)), None
+            ((key_val, k, data[k]) for k in data if re.match(f"^{k}$", key_val)), None
         )
         if result is not None:
             e3.log.debug("%s=%s match %s", key, result[0], result[1])
@@ -200,7 +199,7 @@ class CaseParser:
 
         return value
 
-    def __update_state(self, key: str, value: Any, cursor: Any, prefix: Tuple) -> None:
+    def __update_state(self, key: str, value: Any, cursor: Any, prefix: tuple) -> None:
         """Update state.
 
         :param key: the key to modify. Leading or trailing '+' in the key name
@@ -285,7 +284,7 @@ class CaseParser:
             return cursor
 
 
-def load_with_config(filename: Union[str, List[str]], config: dict) -> Any:
+def load_with_config(filename: str | list[str], config: dict) -> Any:
     """Load yaml config files with case statement handling.
 
     :param filename: a path or list of path. When a list of path
@@ -306,19 +305,17 @@ def load_with_config(filename: Union[str, List[str]], config: dict) -> Any:
             e3.log.debug("load config file: %s", f)
             conf_data = load_ordered(f)
             result = parser.parse(conf_data)
-        except OSError:
-            raise YamlError("cannot read: %s" % f, "load_with_config").with_traceback(
-                sys.exc_info()[2]
-            )
-        except (yaml.parser.ParserError, yaml.constructor.ConstructorError) as e:
+        except OSError as err:
+            raise YamlError(f"cannot read: {f}", "load_with_config") from err
+        except (yaml.parser.ParserError, yaml.constructor.ConstructorError) as err:
             raise YamlError(
-                f"{f} is an invalid yaml file: {e}", "load_with_config"
-            ).with_traceback(sys.exc_info()[2])
+                f"{f} is an invalid yaml file: {err}", "load_with_config"
+            ) from err
 
     return result
 
 
-def load_with_regexp_table(filename: str, selectors: List[str], data: dict) -> dict:
+def load_with_regexp_table(filename: str, selectors: list[str], data: dict) -> dict:
     """Load a yaml file using regexp tables.
 
     :param filename: the yaml file to load
@@ -345,27 +342,27 @@ def load_with_regexp_table(filename: str, selectors: List[str], data: dict) -> d
     with open(filename) as f:
         conf_data = yaml.load(f.read(), OrderedDictYAMLLoader)
 
-    assert isinstance(conf_data, dict), (
-        "top level object in %s should be a dict" % filename
-    )
+    assert isinstance(
+        conf_data, dict
+    ), f"top level object in {filename} should be a dict"
 
     result = {}
 
     for key in conf_data:
         key_data = conf_data[key]
-        assert isinstance(key_data, list), "value for key %s is not a list" % key
+        assert isinstance(key_data, list), f"value for key {key} is not a list"
 
         for line in key_data:
-            assert isinstance(line, list), (
-                "value for key %s should be a list of list" % key
-            )
+            assert isinstance(
+                line, list
+            ), f"value for key {key} should be a list of list"
             assert len(line) == len(selectors) + 1
 
             has_matched = True
             for index, r in enumerate(line[0:-1]):
                 if len(r) == 0:
                     r = ".*"
-                if not re.search(r"^%s$" % r, str(selectors[index])):
+                if not re.search(rf"^{r}$", str(selectors[index])):
                     has_matched = False
 
             if has_matched:
